#!/bin/bash
# Copyright (C) 2007-2009 Timesys Corporation

if [ -z ${WORKORDER} -o -z ${KERNEL_CONFIG} ]; then
  echo "This script should not be called directly run: make checktool-$(basename $0 .sh | cut -d - -f 2-)"
  exit 1
fi

error() {
  ERRORS="${ERRORS}  $*"$'\n'
  ANY_ERRORS=y
}

check() {

ERRORS=
for PARAM in "$@"; do
  NAME=`echo ${PARAM} | cut -d = -f 1`
  VALUE=`echo ${PARAM} | cut -d = -f 2-`

  if [ ${VALUE} == "n" ]; then
    if egrep "^${NAME}=" ${CONFIG} > /dev/null; then
      error "${NAME} is selected (must be unselected)"
    fi
  elif ! egrep "^${NAME}=${VALUE}$" ${CONFIG} > /dev/null; then
    if [ ${VALUE} == "y" ]; then
      # special logic to handle modules
      if egrep "^${NAME}=m$" ${CONFIG} > /dev/null; then
        error "${NAME} is selected as module.  You will need to load this module on the target."
      else
        error "${NAME} is not selected"
      fi
    else
      error "${NAME} does not match pattern: ${VALUE}"
    fi
  fi
done

[ "${ERRORS}" ]
return $?
}

# This series of functions for busybox applet checks must be used in a
# specific order:
# 
# CONFIG=${WORKORDER}
# check_bbox
# ...
# check_bbox_app PKG_NAME APP1 APP2...
# < may call check_bbox_app multiple times >
# if [ "${BBERRORS}" ]; then
#  echo "${BBERRORS}"
# fi
# ...
# if check_bbox_config; then
#  echo ${ERRORS}
# fi
#
check_bbox() {
if check TSWO_SOFTWARE_busybox=y; then
  BUSYBOX=n
else
  BUSYBOX=y
fi
}

check_bbox_app() {
  CONFIG_VALUE=$1
  shift
  BB_APP=$*

  if [[ "${CONFIG_VALUE}" != TSWO_* ]]; then
    CONFIG_VALUE=TSWO_SOFTWARE_${CONFIG_VALUE}
  fi

  if check ${CONFIG_VALUE}=y; then
    if [ ${BUSYBOX} == n ]; then
      BBERRORS="${BBERRORS}  TSWO_SOFTWARE_busybox or ${CONFIG_VALUE} must be selected"$'\n'
    else
      for i in ${BB_APP}; do 
        BUSYBOX_CHECK="${BUSYBOX_CHECK} CONFIG_${i}=y"
      done
    fi
  fi
}

check_bbox_config() {
  CONFIG=${BB_CONFIG}
  check ${BUSYBOX_CHECK}
  return $?
}
