#!/bin/bash
# Copyright (C) 2007-2010 Timesys Corporation

BUILDDIRS=$(ls | grep build_)

if [ -z "$BUILDDIRS" ]; then
    echo "Nothing to clean."
    exit
fi

for builddir in $BUILDDIRS
do
    echo "Cleaning directory $builddir"
    if [ ! -d ${builddir} -o ! -e ${builddir}/toolchain-initial-*.tgz ]; then
        if [ ! -d ${builddir} ]; then
            echo "Error: No build directory to reset."
        fi
        if [ ! -e ${builddir}/toolchain-initial-*.tgz ]; then
            echo "Error: No toolchain from which to reset to."
        fi

        echo ""
        echo "$0"
        echo ""
        echo "Resets the build environment back to a pristine toolchain"
        echo "that does not contain any additional package libraries"
        echo "and includes.  Only useful after make toolchain has completed."
        continue
    fi


    echo "Cleaning package directories..."
    for pkgdir in $(find ${builddir}/ -mindepth 1 -maxdepth 1 -type d | egrep -v 'gcc|cygwin|kernel|linux-2|kernel-headers|fake|mtd-utils|squashfs|cramfs|tsrpm|u-boot|opxml|toolchain|binutils|ccache|gdb-|genext2fs|intltool|glibc|uClibc|pkgconfig|packages|images|oprofile|gmp|mpfr'); do rm -rf $pkgdir;done

    # misc stuff that is not covered above
    rm -rf ${builddir}/gmp-*/.stamp_*host_installed
    rm -rf ${builddir}/mpfr-*/.stamp_*host_installed
    rm -rf ${builddir}/binutils-*/.stamp_libbfd*
    rm -rf ${builddir}/binutils-*/.stamp_libiberty*
    rm -rf ${builddir}/binutils-*/libbfd_host_build
    rm -rf ${builddir}/binutils-*/libiberty_host_build
    rm -rf ${builddir}/libfakekey-*/
    rm -rf ${builddir}/util-linux-*/
    rm -rf ${builddir}/packages/*
    find ${builddir}/fake/{lib,bin}/ ! -name '*fake*' ! -type d -exec rm -f {} \;
    rm -rf ${builddir}/fake/{include,share}
    rm -rf ${builddir}/toolchain-final-*.tgz


    echo "Resetting toolchain"
    (cd ${builddir}/toolchain && rm -rf * && tar xzf ../toolchain-initial-*.tgz)

    echo "Distcleaning RFS"
    make rfs-distclean

done

echo "Done."

