#!/bin/bash

PKG=$1
CONFIGIN=$(find target/ toolchain -wholename "*/$PKG/Config.in" | head -n1)
CATEGORY=$(echo $CONFIGIN | cut -f3 -d\/)
GLOBAL_LOCATION=$(grep TSWO_GLOBAL_SOURCE_LOCATION= .config | cut -f2 -d\")

VERSION=$(grep _${PKG}_VERSION= .config | cut -f2 -d\")
URL=$(grep _${PKG}_URL= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g" -e "s|\\$\(TSWO_GLOBAL_SOURCE_LOCATION\)|$GLOBAL_LOCATION|g")
SOURCE=$(grep _${PKG}_SOURCE= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
PATCHES=$(grep _${PKG}_PATCHES= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
FILES=$(grep _${PKG}_FILES= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
DEPENDS=$(grep _${PKG}_DEPENDS= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
RUNTIME_DEPENDS=$(grep _${PKG}_RUNTIME_DEPENDS= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
CONFIGURE_OPTIONS=$(grep _${PKG}_CONFIGURE_OPTIONS= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
BUILD_OPTIONS=$(grep _${PKG}_BUILD_OPTIONS= .config | cut -f2 -d\" | sed -re "s/\\$\(TSWO_${PKG}_VERSION\)/$VERSION/g")
LICENSE=$(egrep '^# [^ ]+ license: ' target/software/*/$PKG/Config.in 2>/dev/null | cut -f2 -d:| sed -re 's/^ //')
if [ -z "$LICENSE" ]; then
  LICENSE=$(egrep '^# [^ ]+ license: ' toolchain/$PKG/Config.in 2>/dev/null | cut -f2 -d:| sed -re 's/^ //')
fi
if [ -z "$LICENSE" ]; then
  LICENSE=$(egrep '^# [^ ]+ license: ' toolchain/*/$PKG/Config.in 2>/dev/null | cut -f2 -d:| sed -re 's/^ //')
fi

if [ -z "$VERSION" ]; then
  echo "No such package: $PKG"
  exit 1
fi

printf "Package:            %s\n" "$PKG"
printf "Version:            %s\n" "$VERSION"
printf "Category:           %s\n" "$CATEGORY"
printf "License:            %s\n" "$LICENSE"
printf "Url:                %s\n" "$URL"
printf "Source:             %s\n" "$URL/$SOURCE"
printf "Patches:            \n"
for p in $PATCHES
do
printf "                    %s\n" "$URL/$p"
done
printf "Files:              \n"
for f in $FILES
do
printf "                    %s\n" "$URL/$f"
done
printf "Depends:            \n"
for d in $DEPENDS
do
printf "                    %s\n" "$d"
done
printf "Runtime_depends:    \n"
for rd in $RUNTIME_DEPENDS
do
printf "                    %s\n" "$rd"
done
printf "Configure options:  %s\n" "$CONFIGURE_OPTIONS"
printf "Build options:      %s\n" "$BUILD_OPTIONS"
printf "Description:        \n"
./bin/describe_pkg.sh $PKG
