# Copyright (C) 2007-2009 Timesys Corporation
# klibc rules

# Define common macros for klibc from include/common.mk
# This must come after declaring any variables.
KLIBCARCH=$(strip $(call UNQUOTE_VAR,$(TSWO_klibc_ARCH)))

$(eval $(call COMMON_SETUP,klibc))


# add to early-userspace list
ifeq (y,$(TSWO_EARLY_USERSPACE_klibc))
EARLY_USERSPACE+=klibc
endif

ifeq (y,$(TSWO_klibc_FULL_INSTALL))
klibc-rfs-install: klibc-full-install
endif

$(klibc_SRC_DL_DIR)/klibc_config:
	$(call PRINT_INFO,fetching klibc configuration)
	$(call FETCH_REMOTE,$(klibc_SRC_DL_DIR),$(TSWO_klibc_CONFIG))
	cp -f $(klibc_SRC_DL_DIR)/$(notdir $(call UNQUOTE_VAR,$(TSWO_klibc_CONFIG))) $@

$(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_configured: $(klibc_SRC_DL_DIR)/klibc_config
	# copy the downloaded .config
	cp $(klibc_SRC_DL_DIR)/klibc_config $($(PKG)_DDIR)/.config
	# link to the kernel headers in the host area.
	ln -sfn $(TSWO_HOST_INSTALL_DIR) $($(PKG)_DDIR)/linux
	@touch $(@)

$(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_rfs_installed:
	$(call PRINT_INFO,installing klibc $(klibc_VERSION) to early userspace)	
	$(call INSTALL_EARLY_USERSPACE,file,/bin/kinit,$(klibc_DDIR)/usr/kinit/kinit,755,0,0)
	$(call REGISTER_EARLY_USERSPACE,dir,/bin,0755,0,0)
	$(call REGISTER_EARLY_USERSPACE,dir,/lib,0755,0,0)
	$(call REGISTER_EARLY_USERSPACE,dir,/sbin,0755,0,0)
ifeq (y,$(TSWO_EARLY_USERSPACE_BASIC))
	$(call REGISTER_EARLY_USERSPACE,slink,/init,/bin/kinit,0755,0,0)
endif
	@touch $@

.PHONY: klibc-full-install
klibc-full-install: $(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_full_installed

# right now we install static everything, which is probably a bit of a waste
KLIBC_FULL_FILES=usr/dash/sh \
	$(foreach util,cat chroot cpio dd false halt insmod kill ln \
		minips mkdir mkfifo mknod mount nuke pivot_root readlink \
		sleep true umount uname, \
		usr/utils/static/$(util)) \
	$(foreach util,fstype ipconfig nfsmount resume run-init,\
		usr/kinit/$(util)/static/$(util)) \
	$(foreach util,gunzip gzip zcat,usr/gzip/$(util))

$(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_full_installed: $(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_rfs_installed
	$(foreach file,$(KLIBC_FULL_FILES),$(call INSTALL_EARLY_USERSPACE,file,/bin/$(notdir $(file)),$(klibc_DDIR)/$(file),755,0,0))
	$(call INSTALL_EARLY_USERSPACE,file,/sbin/hotplug,$(klibc_SRC_DL_DIR)/hotplug,755,0,0)
	$(call INSTALL_EARLY_USERSPACE,file,/arch.cmd,$(klibc_SRC_DL_DIR)/arch.cmd,755,0,0)
	@touch $@

$(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_host_installed: $(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_pre_host_installed
	$(call PRINT_INFO,installing klibc $(klibc_VERSION) to host)
	$(MAKE) -C $(klibc_DDIR) install $(klibc_BUILD_OPTIONS) INSTALLROOT=$(TSWO_HOST_INSTALL_DIR) 
	@touch $@

$(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_packaged: $(BUILD_DIR)/klibc-$(klibc_VERSION)/.stamp_pre_packaged
	@touch $@

# vim:set noexpandtab filetype=make: 
