#!/bin/bash

# Actually this should point to the first line that isn't script
LINES_IN_SCRIPT=50

for prog in tail tar gzip; do
  if ! which ${prog} > /dev/null; then
     echo "Error: Required program '${prog}' not found"
     exit 1
  fi
done

if [ -z $TMP_DIR ]; then
   TMP_DIR=/tmp
fi

MY_TMP=${TMP_DIR}/self-extract-$$

mkdir ${TMP_DIR}/self-extract-$$
if [ $? != 0 ]; then
   echo "Error: could not create temporary directory"
   exit 1
fi

tail --lines=+$LINES_IN_SCRIPT $0 | gzip -d | tar --no-same-owner -x -C ${MY_TMP}
if [ $? != 0 ]; then
   echo "Error: Unpack of archive failed (partial unpack in ${MY_TMP})"
   exit 1
fi

trap "printf 'Cleaning up...'; rm -rf $MY_TMP; echo Done; exit 1" SIGINT SIGTERM

if [ -x ${MY_TMP}/install ]; then
   pushd ${MY_TMP} > /dev/null
   ./install
   RESULT=$?
   popd > /dev/null
   rm -rf ${MY_TMP}
   if [ $? != 0 ]; then
      echo "Error: Could not remove temporary directory after install"
      exit 1
   fi
   exit ${RESULT}
fi

# No installer, just report temp location to user.
echo "Archive unpacked into ${MY_TMP}"
exit 0
## END OF SCRIPT ##
