# Copyright (C) 2007-2009 Timesys Corporation
# kernel make rules

# Define common macros for this package from include/common.mk
#$(eval $(call COMMON_SETUP,kernel))
# we don't want the default macros here... too much customizing
$(eval $(call MAP_CONFIG_VARS_TO_MK_VARS,kernel))

kernel_EXTRAVERSION=$(strip $(call UNQUOTE_VAR,$(TSWO_kernel_EXTRAVERSION)))

kernel_BUILDNUM?=1
kernel_SRC_DL_DIR=$(SRC_DL_DIR)/k/kernel/kernel-$(kernel_VERSION)
kernel_PKG_INSTALL_DIR=$(PACKAGE_DIR)/kernel_install
kernel_PKG_NAME=$(PACKAGE_DIR)/kernel-$(kernel_VERSION)-$(TSWO_FULL_TARGET_NAME)-$(kernel_BUILDNUM).tgz
FINAL_KERNEL_FILENAME=$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_FILE)))
FINAL_KERNEL_SHORTFILENAME=$(notdir $(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_FILE))))

ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel_WORKINGDIR=$(BUILD_DIR)/linux-$(kernel_VERSION)
else
kernel_WORKINGDIR=$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_EXTERNAL_SRC)))
endif

# Early userspace support
EARLY_USERSPACE_DIR=$(kernel_WORKINGDIR)/early-userspace
EARLY_USERSPACE_TAB=$(kernel_WORKINGDIR)/early-userspace.tab

# extra blank line at end of these macros is necessary for use in foreach
define INSTALL_EARLY_USERSPACE
	mkdir -p $(EARLY_USERSPACE_DIR)
	cp -f $(3) $(EARLY_USERSPACE_DIR)
	$(call REGISTER_EARLY_USERSPACE,$(1),$(2),$(EARLY_USERSPACE_DIR)/$(notdir $(3)),$(4),$(5),$(6),$(7))

endef

define REGISTER_EARLY_USERSPACE
	@echo " REGISTER:" $(1)$$'\t'$(2)$$'\t'$(notdir $(3))$$'\t'$(4)$$'\t'$(5)$$'\t'$(6)$$'\t'$(7)$$'\t'$(8)
	@touch $(EARLY_USERSPACE_TAB)
	@grep -v $$'\t'"$(2)"$$'\t' $(EARLY_USERSPACE_TAB) > $(EARLY_USERSPACE_TAB).tmp || true
	@mv -f $(EARLY_USERSPACE_TAB).tmp $(EARLY_USERSPACE_TAB)
	@echo $(1)$$'\t'$(2)$$'\t'$(3)$$'\t'$(4)$$'\t'$(5)$$'\t'$(6)$$'\t'$(7)$$'\t'$(8) >> $(EARLY_USERSPACE_TAB)

endef

.PHONY: early-userspace
# early-userspace is defined in the top-level makefile because of evaluation
# order problems.

ifeq (y,$(TSWO_EARLY_USERSPACE_ALL_MODULES))
MODULE_LIST=$(shell cd $(@D) && find -name "*.ko")
else
ifneq ("",$(TSWO_EARLY_USERSPACE_MODULE_LIST))
MODULE_LIST=$(call UNQUOTE_VAR,$(TSWO_EARLY_USERSPACE_MODULE_LIST))
endif
endif

ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
$(kernel_WORKINGDIR)/.stamp_pre_early_userspace: $(kernel_WORKINGDIR)/.stamp_unpacked
endif
$(kernel_WORKINGDIR)/.stamp_pre_early_userspace:
	$(call PRINT_INFO,early userspace setup)
	@mkdir -p $(EARLY_USERSPACE_DIR)
	$(call REGISTER_EARLY_USERSPACE,dir,/proc,0755,0,0)
	$(call REGISTER_EARLY_USERSPACE,dir,/sys,0755,0,0)
	$(call REGISTER_EARLY_USERSPACE,dir,/dev,0755,0,0)
	$(call REGISTER_EARLY_USERSPACE,nod,/dev/console,0600,0,0,c,5,1)
	$(call REGISTER_EARLY_USERSPACE,nod,/dev/null,0666,0,0,c,1,3)
ifdef MODULE_LIST
	$(call REGISTER_EARLY_USERSPACE,dir,/lib,0755,0,0)
	$(call REGISTER_EARLY_USERSPACE,dir,/lib/modules,0755,0,0)
	$(foreach module,$(MODULE_LIST),$(call REGISTER_EARLY_USERSPACE,file,/lib/modules/$(notdir $(module)),$(@D)/$(module),0644,0,0))
endif
	@touch $(@)
$(kernel_WORKINGDIR)/.stamp_post_early_userspace:
	sort -k 2 $(EARLY_USERSPACE_TAB) > $(EARLY_USERSPACE_TAB).tmp
	mv -f $(EARLY_USERSPACE_TAB).tmp $(EARLY_USERSPACE_TAB)
	@touch $(@)

ifdef MODULE_LIST
$(kernel_WORKINGDIR)/.stamp_pre_early_userspace: $(kernel_WORKINGDIR)/.stamp_modules_built
endif

ifeq (y,$(TSWO_EARLY_USERSPACE))
$(kernel_WORKINGDIR)/.stamp_image_built: early-userspace
kernel_INITRAMFS_SOURCE=$(EARLY_USERSPACE_TAB)
endif
ifeq (y,$(strip $(TSWO_RFS_FS_cpio_INITRAMFS)))
kernel_INITRAMFS_SOURCE=$(IMAGES_DIR)/rfs/rootfs.cpio.gz
endif


KERNEL_CONFIG=
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
KERNEL_CONFIG=$(kernel_SRC_DL_DIR)/kernel_config
else
ifeq (y,$(strip $(TSWO_KERNEL_EXTERNAL_USE_CONFIGURATION)))
KERNEL_CONFIG=$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_EXTERNAL_CONFIG)))
else
KERNEL_CONFIG=$(kernel_WORKINGDIR)/.config
endif
endif

# Define custom targets that can be used by the standard targets
# for pre and post actions.
$(kernel_SRC_DL_DIR)/kernel_config:
	$(call PRINT_INFO,fetching kernel config)
	$(call FETCH_REMOTE,$(kernel_SRC_DL_DIR),$(TSWO_KERNEL_CONFIG))
	@if [ ! -f $@ ]; then mv -f $(kernel_SRC_DL_DIR)/$(notdir $(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_CONFIG)))) $@;fi
	@touch $@


$(kernel_SRC_DL_DIR)/$(kernel_SOURCE):
	$(call PRINT_INFO,fetching kernel)
	@mkdir -p $(kernel_SRC_DL_DIR)
	$(call FETCH_REMOTE,$(kernel_SRC_DL_DIR),$(kernel_URL)/$(kernel_SOURCE))
	$(foreach patch,$(kernel_PATCHES),$(call FETCH_REMOTE,$(kernel_SRC_DL_DIR),$(kernel_URL)/$(patch)))
	@touch $@

$(kernel_WORKINGDIR)/.stamp_unpacked: $(kernel_SRC_DL_DIR)/$(kernel_SOURCE)
	$(call PRINT_INFO,extracting linux-$(kernel_VERSION))
	@mkdir -p $(BUILD_DIR)
	@$(INFLATE$(suffix $(kernel_SOURCE))) $(kernel_SRC_DL_DIR)/$(kernel_SOURCE) | \
		$(TAR) -C $(BUILD_DIR) $(TSWO_TAR_OPTIONS) -
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_patched: $(kernel_WORKINGDIR)/.stamp_unpacked
	$(foreach patch,$(kernel_PATCHES),$(call APPLY_PATCH,$(@D),$(kernel_SRC_DL_DIR)/$(patch)))
	@$(SED) -i "s,^EXTRAVERSION =.*,EXTRAVERSION = $(kernel_EXTRAVERSION)," $(@D)/Makefile
	@touch $(@)

kernel-fetch: $(kernel_SRC_DL_DIR)/$(kernel_SOURCE) $(KERNEL_CONFIG)
kernel-unpack: kernel-fetch $(kernel_WORKINGDIR)/.stamp_unpacked
kernel-patch: kernel-unpack $(kernel_WORKINGDIR)/.stamp_patched



ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
$(BUILD_DIR)/kernel-headers/.stamp_unpacked: $(kernel_WORKINGDIR)/.stamp_patched
endif
$(BUILD_DIR)/kernel-headers/.stamp_unpacked:
	$(call PRINT_INFO,unpacking kernel-headers)
	@if [ -d $(BUILD_DIR)/kernel-headers ]; then rm -rf $(BUILD_DIR)/kernel-headers;fi
	@cp -r $(kernel_WORKINGDIR) $(BUILD_DIR)/kernel-headers
	@touch $(@)

$(BUILD_DIR)/kernel-headers/.stamp_patched: $(BUILD_DIR)/kernel-headers/.stamp_unpacked
	@touch $(@)

$(BUILD_DIR)/kernel-headers/.stamp_host_installed: $(BUILD_DIR)/kernel-headers/.stamp_patched
	$(call PRINT_INFO,installing kernel-headers for host)
	mkdir -p $(TSWO_HOST_INSTALL_DIR)/include/
	cd $(BUILD_DIR)/kernel-headers && PATH=$(BUILD_PATH) \
		$(MAKE) headers_install \
		ARCH=$(TSWO_KERNEL_HEADER_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		INSTALL_HDR_PATH=$(@D)/kernel_headers_inst
	cp -rf $(@D)/kernel_headers_inst/include/* $(TSWO_HOST_INSTALL_DIR)/include/
	@touch $(@)

$(BUILD_DIR)/kernel-headers/.stamp_rfs_installed: $(BUILD_DIR)/kernel-headers/.stamp_unpacked
	$(call PRINT_INFO,installing kernel-headers into RFS)
	mkdir -p $(TSWO_RFS_INSTALL_DIR)/usr/include/
	cd $(BUILD_DIR)/kernel-headers && PATH=$(BUILD_PATH) \
		$(MAKE) headers_install \
		ARCH=$(TSWO_KERNEL_HEADER_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		INSTALL_HDR_PATH=$(@D)/kernel_headers_inst
	cp -rf $(@D)/kernel_headers_inst/include/* $(TSWO_RFS_INSTALL_DIR)/usr/include/
	@touch $(@)

kernel_headers_PKG_INSTALL_DIR=$(PACKAGE_DIR)/kernel-headers_install
kernel_headers_PKG_NAME=$(PACKAGE_DIR)/kernel-headers-$(kernel_VERSION)-$(TSWO_FULL_TARGET_NAME)-$(kernel_BUILDNUM).tgz
$(BUILD_DIR)/kernel-headers/.stamp_packaged: $(BUILD_DIR)/kernel-headers/.stamp_unpacked
	$(call PRINT_INFO,packaging kernel-headers)
	mkdir -p $(kernel_headers_PKG_INSTALL_DIR)/usr/include/
	cd $(BUILD_DIR)/kernel-headers && PATH=$(BUILD_PATH) \
		$(MAKE) headers_install \
		ARCH=$(TSWO_KERNEL_HEADER_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		INSTALL_HDR_PATH=$(@D)/kernel_headers_inst
	cp -rf $(@D)/kernel_headers_inst/include/* $(kernel_headers_PKG_INSTALL_DIR)/usr/include/
	$(call CREATE_PACKAGE, kernel-headers,$(kernel_headers_PKG_INSTALL_DIR),$(kernel_headers_PKG_NAME),$(kernel_VERSION),$(kernel_BUILDNUM),)
	@rm -rf $(kernel_headers_PKG_INSTALL_DIR)
	@touch $(@)

host-kernel-headers: $(BUILD_DIR)/kernel-headers/.stamp_host_installed
rfs-kernel-headers: $(BUILD_DIR)/kernel-headers/.stamp_rfs_installed
kernel-headers-package: $(BUILD_DIR)/kernel-headers/.stamp_packaged

$(kernel_WORKINGDIR)/.stamp_register_initramfs_source: $(KERNEL_CONFIG)
	$(call PRINT_INFO,configuring linux-$(kernel_VERSION) initramfs source)
	$(SED) -i -r -e "s,^.*(CONFIG_BLK_DEV_INITRD).*$$,\1=y," $(@D)/.config $(KERNEL_CONFIG)
	$(SED) -i -r -e "s,^.*(CONFIG_INITRAMFS_SOURCE).*$$,\1=\"$(kernel_INITRAMFS_SOURCE)\"," $(@D)/.config $(KERNEL_CONFIG)
	for f in $(@D)/.config $(KERNEL_CONFIG);do \
		if [ -z "$$(grep CONFIG_INITRAMFS_SOURCE $$f)" ]; then \
			echo 'CONFIG_INITRAMFS_SOURCE="$(kernel_INITRAMFS_SOURCE)"' >> $$f; \
		fi; \
	done
	@yes '' | PATH=$(BUILD_PATH) $(MAKE) KBUILD_HAVE_NLS=no -C $(@D) \
		ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		oldconfig
	@touch $(@)

KERNEL_CONFIGURE_DEPS=
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
KERNEL_CONFIGURE_DEPS=kernel-patch $(kernel_WORKINGDIR)/.stamp_configured
else
ifeq (y,$(strip $(TSWO_KERNEL_EXTERNAL_USE_CONFIGURATION)))
KERNEL_CONFIGURE_DEPS=$(kernel_WORKINGDIR)/.stamp_external_configured
endif
endif
kernel-configure: $(KERNEL_CONFIGURE_DEPS)

$(kernel_WORKINGDIR)/.stamp_configured: $(KERNEL_CONFIG)
	$(call PRINT_INFO,configuring linux-$(kernel_VERSION))
	cp -f $(kernel_SRC_DL_DIR)/kernel_config $(@D)/.config
ifneq (,$(findstring cygwin,$(TSWO_HOST_NAME)))
	$(SED) -i -r -e "s,^(HOSTCFLAGS\W+=.*)$$,\1 -DKBUILD_NO_NLS," \
		$(@D)/Makefile
endif
	@yes '' | PATH=$(BUILD_PATH) $(MAKE) KBUILD_HAVE_NLS=no -C $(@D) \
		ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		oldconfig
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_external_configured: $(KERNEL_CONFIG)
	$(call PRINT_INFO,configuring linux-$(kernel_VERSION) external kernel with external configuration)
	cp -f $(KERNEL_CONFIG) $(@D)/.config
	yes '' | PATH=$(BUILD_PATH) $(MAKE) KBUILD_HAVE_NLS=no -C $(@D) \
		ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		oldconfig
	@touch $(@)

KERNEL_BUILD_DEPS=kernel-configure
ifeq (uImage,$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_FILE))))
KERNEL_BUILD_DEPS+=mkimage-host-install
endif
KERNEL_BUILD_DEPS+=kernel-build-image kernel-build-modules
ifeq (y,$(strip $(TSWO_KERNEL_DEVICE_TREE)))
KERNEL_BUILD_DEPS+=kernel_devicetree-build
endif
ifeq (y,$(TSWO_EARLY_USERSPACE))
KERNEL_BUILD_DEPS+=$(kernel_WORKINGDIR)/.stamp_register_initramfs_source
endif
ifeq (y,$(TSWO_RFS_FS_cpio_INITRAMFS))
KERNEL_BUILD_DEPS+=kernel-build-image_with_initramfs
endif

kernel-build: $(KERNEL_BUILD_DEPS)
kernel-build-image: kernel-configure $(kernel_WORKINGDIR)/.stamp_image_built
kernel-build-modules: kernel-configure $(kernel_WORKINGDIR)/.stamp_modules_built
kernel-build-image_with_initramfs: rfs-images kernel-configure $(kernel_WORKINGDIR)/.stamp_image_with_initramfs_built

$(kernel_WORKINGDIR)/.stamp_image_built: $(KERNEL_CONFIG)
	$(call PRINT_INFO,building linux-$(kernel_VERSION) image)
	PATH=$(BUILD_PATH) $(MAKE) -C $(@D) \
		ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		$(TSWO_KERNEL_TARGET)
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_modules_built: $(KERNEL_CONFIG)
	$(call PRINT_INFO,building linux-$(kernel_VERSION) modules)
	if fgrep -q CONFIG_MODULES=y $(@D)/.config; then \
		PATH=$(BUILD_PATH) $(MAKE) -C $(@D) \
			ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
			modules; \
	fi
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_image_with_initramfs_built: $(KERNEL_CONFIG) $(kernel_WORKINGDIR)/.stamp_register_initramfs_source $(IMAGES_DIR)/rfs/rootfs.cpio.gz
	$(call PRINT_INFO,building linux-$(kernel_VERSION) image with an initramfs)
	PATH=$(BUILD_PATH) $(MAKE) -C $(@D) \
		ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		$(TSWO_KERNEL_TARGET)
	@touch $(@)

KERNEL_RFS_INSTALL_DEPS=kernel-configure
ifneq (y,$(TSWO_RFS_FS_cpio_INITRAMFS))
KERNEL_RFS_INSTALL_DEPS+=kernel-build-image
endif
KERNEL_RFS_INSTALL_DEPS+=kernel-build-modules
ifeq (y,$(TSWO_INSTALL_KERNEL_TO_TARGET))
KERNEL_RFS_INSTALL_DEPS+=$(kernel_WORKINGDIR)/.stamp_rfs_installed
endif
ifeq (y,$(TSWO_INSTALL_KERNEL_DEBUG_TO_TARGET))
KERNEL_RFS_INSTALL_DEPS+=$(kernel_WORKINGDIR)/.stamp_debug_rfs_installed
endif
ifeq (y,$(TSWO_INSTALL_KERNEL_MODULES_TO_TARGET))
KERNEL_RFS_INSTALL_DEPS+=$(kernel_WORKINGDIR)/.stamp_modules_rfs_installed
endif

kernel-rfs-install: $(KERNEL_RFS_INSTALL_DEPS)

$(kernel_WORKINGDIR)/.stamp_modules_rfs_installed: $(KERNEL_CONFIG)
	$(call PRINT_INFO,install linux-$(kernel_VERSION) modules to the RFS)
	if fgrep -q CONFIG_MODULES=y $(@D)/.config; then \
		PATH=$(BUILD_PATH) $(MAKE) -C $(@D) \
			ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
			INSTALL_MOD_PATH=$(TSWO_RFS_INSTALL_DIR) \
			modules_install; \
	fi
	@rm -f $(TSWO_RFS_INSTALL_DIR)/lib/modules/*/source
	@rm -f $(TSWO_RFS_INSTALL_DIR)/lib/modules/*/build
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_rfs_installed: $(KERNEL_CONFIG)
	$(call PRINT_INFO,install linux-$(kernel_VERSION) to the RFS)
	@mkdir -p $(TSWO_RFS_INSTALL_DIR)/boot
	@if [ -f $(@D)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) ]; then cp -f $(@D)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) $(TSWO_RFS_INSTALL_DIR)/boot/$(FINAL_KERNEL_SHORTFILENAME); else cp -f $(@D)/arch/$(TSWO_KERNEL_ARCH)/boot/$(FINAL_KERNEL_FILENAME) $(TSWO_RFS_INSTALL_DIR)/boot/$(FINAL_KERNEL_SHORTFILENAME); fi
	@cp -f $(@D)/.config $(TSWO_RFS_INSTALL_DIR)/boot/config-$(kernel_VERSION)$(kernel_EXTRAVERSION)
	@cp -f $(@D)/System.map $(TSWO_RFS_INSTALL_DIR)/boot/$(kernel_VERSION)$(kernel_EXTRAVERSION)-System.map
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_debug_rfs_installed:
	$(call PRINT_INFO,install vmlinux to the RFS)
	@mkdir -p $(TSWO_RFS_INSTALL_DIR)/boot
	@if [ -f $(@D)/vmlinux ]; then cp -f $(@D)/vmlinux $(TSWO_RFS_INSTALL_DIR)/boot/vmlinux; fi
	@touch $(@)

kernel-host-install: kernel-install-image 
ifeq (y,$(TSWO_KERNEL_DEVICE_TREE))
kernel-host-install: kernel_devicetree-host-install
endif

kernel-install-image: $(KERNEL_CONFIG) kernel-build $(IMAGES_DIR)/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION)
$(IMAGES_DIR)/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION):
	$(call PRINT_INFO,install linux-$(kernel_VERSION) to the host)
	@mkdir -p $(IMAGES_DIR)
	@if [ -f $(kernel_WORKINGDIR)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) ]; then cp -f $(kernel_WORKINGDIR)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) $(IMAGES_DIR)/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION); else cp -f $(kernel_WORKINGDIR)/arch/$(TSWO_KERNEL_ARCH)/boot/$(FINAL_KERNEL_FILENAME) $(IMAGES_DIR)/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION); fi

kernel-package: $(KERNEL_CONFIG) kernel-build $(kernel_WORKINGDIR)/.stamp_packaged
$(kernel_WORKINGDIR)/.stamp_packaged: $(KERNEL_CONFIG)
ifeq (y,$(TSWO_KERNEL_DEVICE_TREE))
$(kernel_WORKINGDIR)/.stamp_packaged: $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb
endif
$(kernel_WORKINGDIR)/.stamp_packaged:
	$(call PRINT_INFO,packaging kernel)
	@mkdir -p $(kernel_PKG_INSTALL_DIR)
	if fgrep -q CONFIG_MODULES=y $(@D)/.config; then \
		PATH=$(BUILD_PATH) $(MAKE) -C $(@D) \
			ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
			INSTALL_MOD_PATH=$(kernel_PKG_INSTALL_DIR) \
			modules_install; \
	fi
	@rm -f $(kernel_PKG_INSTALL_DIR)/lib/modules/*/source
	@rm -f $(kernel_PKG_INSTALL_DIR)/lib/modules/*/build
	@mkdir -p $(kernel_PKG_INSTALL_DIR)/boot
	@if [ -f $(@D)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) ]; then cp -f $(@D)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) $(kernel_PKG_INSTALL_DIR)/boot/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION); else cp -f $(@D)/arch/$(TSWO_KERNEL_ARCH)/boot/$(FINAL_KERNEL_FILENAME) $(kernel_PKG_INSTALL_DIR)/boot/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION); fi
	@cp -f $(@D)/.config $(kernel_PKG_INSTALL_DIR)/boot/config-$(kernel_VERSION)$(kernel_EXTRAVERSION)
	@cp -f $(@D)/System.map $(kernel_PKG_INSTALL_DIR)/boot/$(kernel_VERSION)$(kernel_EXTRAVERSION)-System.map
ifeq (y,$(TSWO_INSTALL_KERNEL_DEBUG_TO_TARGET))
	@if [ -f $(@D)/vmlinux ]; then cp -f $(@D)/vmlinux $(kernel_PKG_INSTALL_DIR)/boot/vmlinux; fi
endif
ifeq (y,$(TSWO_KERNEL_DEVICE_TREE))
	@cp -f $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb $(kernel_PKG_INSTALL_DIR)/boot/$(TSWO_BOARD).dtb
endif
	$(call CREATE_PACKAGE, kernel,$(kernel_PKG_INSTALL_DIR),$(kernel_PKG_NAME),$(kernel_VERSION),$(kernel_BUILDNUM),)
	@rm -rf $(kernel_PKG_INSTALL_DIR)
	@touch $(@)

kernel-images-clean:
	@rm -f $(IMAGES_DIR)/$(TSWO_BOARD).dtb
	@rm -f $(IMAGES_DIR)/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION)

kernel-rfs-distclean:
	-@cd $(kernel_WORKINGDIR) && rm -f .stamp*rfs*installed

ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel-clean: kernel-images-clean
	@rm -rf $(kernel_WORKINGDIR)
kernel-distclean: kernel-clean kernel-source-distclean
	@rm -rf $(kernel_SRC_DL_DIR)
else
kernel-clean: kernel-images-clean
	@cd $(kernel_WORKINGDIR) && make clean ARCH=$(TSWO_KERNEL_HEADER_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- && rm -f .stamp_*
kernel-distclean: kernel-images-clean
	@cd $(kernel_WORKINGDIR) && make distclean ARCH=$(TSWO_KERNEL_HEADER_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)-
endif

ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel-%config: kernel-patch
	@cd $(kernel_WORKINGDIR) && \
		if [ -f $(kernel_SRC_DL_DIR)/kernel_config ]; then cp -f $(kernel_SRC_DL_DIR)/kernel_config .config;fi && \
		$(MAKE) \
			ARCH=$(TSWO_KERNEL_ARCH) KBUILD_HAVE_NLS=no CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
			$*config && \
		if [ -f .config ]; then cp -f .config $(kernel_SRC_DL_DIR)/kernel_config;fi
else
kernel-%config:
	@cd $(kernel_WORKINGDIR) && \
		$(MAKE) \
			ARCH=$(TSWO_KERNEL_ARCH) KBUILD_HAVE_NLS=no CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
			$*config
ifeq (y,$(strip $(TSWO_KERNEL_EXTERNAL_USE_CONFIGURATION)))
	@cd $(kernel_WORKINGDIR) && \
		cp -f .config $(KERNEL_CONFIG)
endif
endif


# kernel source
.PHONY: kernel-source-fetch kernel-source-unpack kernel-source-patch kernel-source-configure kernel-source-build kernel-source-check kernel-source-rfs-install kernel-source-host-install kernel-source-clean kernel-source-distclean kernel-images-clean
KERNEL_SRC_PKG_DIR=$(BUILD_DIR)/linux-src-$(kernel_VERSION)/
kernel_SRC_PKG_NAME=$(PACKAGE_DIR)/kernel-source-$(kernel_VERSION)-$(TSWO_FULL_TARGET_NAME).tgz
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel-source-fetch: kernel-fetch
kernel-source-unpack: kernel-fetch
kernel-source-patch: kernel-fetch
kernel-source-configure: kernel-fetch
kernel-source-build: kernel-fetch
kernel-source-check: kernel-fetch
kernel-source-rfs-install: kernel-fetch
kernel-source-host-install: kernel-fetch
endif
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel-source-package: kernel-fetch $(BUILD_DIR)/linux-src-$(kernel_VERSION)/.stamp_src_packaged
else
kernel-source-package: $(BUILD_DIR)/linux-src-$(kernel_VERSION)/.stamp_src_packaged
endif
kernel-source-clean:
	rm -rf $(KERNEL_SRC_PKG_DIR)
kernel-source-distclean: kernel-source-clean

$(BUILD_DIR)/linux-src-$(kernel_VERSION)/.stamp_src_packaged: $(KERNEL_CONFIG)
	$(call PRINT_INFO,packaging kernel-source)
	@rm -rf $(KERNEL_SRC_PKG_DIR)
	@mkdir -p $(KERNEL_SRC_PKG_DIR)
	@mkdir -p $(PACKAGE_DIR)
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
	@$(INFLATE$(suffix $(kernel_SOURCE))) $(kernel_SRC_DL_DIR)/$(kernel_SOURCE) | \
		$(TAR) -C $(KERNEL_SRC_PKG_DIR) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(kernel_PATCHES),$(call APPLY_PATCH,$(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION),$(kernel_SRC_DL_DIR)/$(patch)))
	@$(SED) -i "s,^EXTRAVERSION =.*,EXTRAVERSION = $(kernel_EXTRAVERSION)," $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Makefile
	@cp -f $(kernel_SRC_DL_DIR)/kernel_config $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/.config
else
	@cp -rf $(kernel_WORKINGDIR) $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)
	@rm -rf $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/.stamp_*
	@$(MAKE) -C $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION) ARCH=$(TSWO_KERNEL_ARCH) KBUILD_HAVE_NLS=no CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- clean
endif
	@yes '' | PATH=$(BUILD_PATH) $(MAKE) -C $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION) \
		ARCH=$(TSWO_KERNEL_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		oldconfig
	@rm -f $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/.config.old
	@rm -f $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/scripts/kconfig/{conf,*.o}
	@rm -f $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/scripts/basic/{docproc,fixdep}
	@mkdir -p $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)tsconf
	@echo "Default kernel configuration file for $(TSWO_BOARD)" > $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)tsconf/defconfig.help.xml
	@cp -f $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/.config $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)tsconf/defconfig.kfg
	@echo '<?xml version="1.0" standalone="no"?>' > $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '<bsp>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <shortname>$(strip $(call UNQUOTE_VAR,$(TSWO_BOARD)))</shortname>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <name>$(strip $(call UNQUOTE_VAR,$(TSWO_BOARD)))</name>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <cpu>$(strip $(call UNQUOTE_VAR,$(TSWO_ARCH)))</cpu>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <arch>$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_ARCH)))</arch>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <crosscompile>$(strip $(call UNQUOTE_VAR,$(TSWO_FULL_TARGET_NAME)))-</crosscompile>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <tsrel>kernel-$(strip $(call UNQUOTE_VAR,$(TSWO_kernel_VERSION)))-ts.$(strip $(call UNQUOTE_VAR,$(TSWO_BOARD)))</tsrel>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <configfiles' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    location="$(strip $(call UNQUOTE_VAR,$(TSWO_BOARD)))tsconf"' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    ext=".kfg"' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    helpext=".help.xml">' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  </configfiles>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo ' <nativefstarget' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    build="$(strip $(call UNQUOTE_VAR,$(FINAL_KERNEL_FILENAME)))"' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@if [ -f $(kernel_WORKINGDIR)/arch/$(TSWO_KERNEL_ARCH)/boot/images/$(FINAL_KERNEL_FILENAME) ]; then \
	echo '    image="arch/$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_ARCH)))/boot/images/$(strip $(call UNQUOTE_VAR,$(FINAL_KERNEL_FILENAME)))">' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml; \
		else \
	echo '    image="arch/$(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_ARCH)))/boot/$(strip $(call UNQUOTE_VAR,$(FINAL_KERNEL_FILENAME)))">' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml; \
		fi
	@echo '  </nativefstarget>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <initrdtarget' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    build=""' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    image=""' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    initrddir=""' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    initrdimgname=""' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    size="">' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  </initrdtarget>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  <jffs' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    eblksz=""' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '    extra="">' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '  </jffs>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	@echo '</bsp>' >> $(KERNEL_SRC_PKG_DIR)/linux-$(kernel_VERSION)/Documentation/$(TSWO_BOARD)build.xml
	$(call CREATE_TGZ, $(KERNEL_SRC_PKG_DIR),$(kernel_SRC_PKG_NAME))
	@rm -rf $(kernel_PKG_INSTALL_DIR)
	@touch $@


# kernel device tree (powerpc only)
.PHONY: kernel_devicetree-fetch kernel_devicetree-unpack kernel_devicetree-patch kernel_devicetree-configure kernel_devicetree-build kernel_devicetree-rfs-install kernel_devicetree-package kernel_devicetree-clean kernel_devicetree-distclean

ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel_devicetree-fetch: kernel-fetch $(kernel_SRC_DL_DIR)/kernel_devicetree.dts
kernel_devicetree-patch: kernel-patch kernel_devicetree-unpack
else
kernel_devicetree-fetch: $(kernel_SRC_DL_DIR)/kernel_devicetree.dts
kernel_devicetree-patch: kernel_devicetree-unpack
endif

kernel_devicetree-unpack: kernel_devicetree-fetch
kernel_devicetree-configure: kernel_devicetree-patch

ifeq (y,$(strip $(TSWO_KERNEL_DEVICE_TREE_IN_TREE_BUILD)))
kernel_devicetree-build:
kernel_devicetree-host-install:
kernel_devicetree-rfs-install:
else
kernel_devicetree-build: kernel_devicetree-configure dtc-host-install $(kernel_WORKINGDIR)/.stamp_dtb_built
kernel_devicetree-host-install: kernel_devicetree-build $(kernel_WORKINGDIR)/.stamp_dtb_host_installed
ifeq (y,$(TSWO_INSTALL_KERNEL_TO_TARGET))
kernel_devicetree-rfs-install: kernel_devicetree-build $(kernel_WORKINGDIR)/.stamp_dtb_rfs_installed
else
kernel_devicetree-rfs-install:
endif
endif

kernel_devicetree-package:
kernel_devicetree-clean:
kernel_devicetree-distclean:

ifeq (y,$(strip $(TSWO_KERNEL_DEVICE_TREE_IN_TREE_BUILD)))
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
kernel_devicetree-fetch: kernel-patch $(kernel_WORKINGDIR)/.stamp_dtb_tree_installed
else
kernel_devicetree-fetch: $(kernel_WORKINGDIR)/.stamp_dtb_tree_installed
endif
endif

$(kernel_SRC_DL_DIR)/kernel_devicetree.dts:
	$(call PRINT_INFO,fetching kernel device tree source)
	$(call FETCH_REMOTE,$(kernel_SRC_DL_DIR),$(TSWO_KERNEL_DEVICE_TREE_FILE))
	@if [ ! -f $@ ]; then mv -f $(kernel_SRC_DL_DIR)/$(notdir $(strip $(call UNQUOTE_VAR,$(TSWO_KERNEL_DEVICE_TREE_FILE)))) $@;fi
	@touch $@

$(kernel_WORKINGDIR)/.stamp_dtb_tree_installed:
	$(call PRINT_INFO,copying kernel device tree to kernel tree)
	@cp -r $(kernel_SRC_DL_DIR)/kernel_devicetree.dts $(kernel_WORKINGDIR)/arch/$(TSWO_KERNEL_ARCH)/boot/dts/$(TSWO_KERNEL_DEVICE_TREE_IN_TREE_FILENAME)
	@touch $@

$(kernel_WORKINGDIR)/.stamp_dtb_built: $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb
$(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb: $(kernel_SRC_DL_DIR)/kernel_devicetree.dts
	$(call PRINT_INFO,building kernel device tree blob)
	PATH=$(BUILD_PATH) $(TSWO_HOST_INSTALL_DIR)/bin/dtc -O dtb -S $(TSWO_KERNEL_DEVICE_TREE_SIZE) -o $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb $(kernel_SRC_DL_DIR)/kernel_devicetree.dts
	@touch $@

$(kernel_WORKINGDIR)/.stamp_dtb_host_installed: $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb
	$(call PRINT_INFO,installing kernel device tree blob to the host)
	@mkdir -p $(IMAGES_DIR)
	cp -f $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb $(IMAGES_DIR)/$(TSWO_BOARD).dtb
	@touch $(@)

$(kernel_WORKINGDIR)/.stamp_dtb_rfs_installed: $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb
	$(call PRINT_INFO,installing kernel device tree blob to RFS)
	mkdir -p $(TSWO_RFS_INSTALL_DIR)/boot/
	cp -f $(kernel_WORKINGDIR)/$(TSWO_BOARD).dtb $(TSWO_RFS_INSTALL_DIR)/boot/$(TSWO_BOARD).dtb
	@touch $(@)

# add to packages list
ifneq (y,$(strip $(TSWO_KERNEL_EXTERNAL)))
SOFTWARE+=kernel
endif
ifeq (y,$(TSWO_PACKAGE_KERNEL_SOURCE))
SOFTWARE+=kernel-source
endif
ifeq (y,$(TSWO_KERNEL_DEVICE_TREE))
SOFTWARE+=kernel_devicetree
endif

# vim:set noexpandtab: 
