# Copyright (C) 2007-2010 Timesys Corporation
# evince rules

# Define common macros for evince from include/common.mk
# This must come after declaring any variables.
$(eval $(call COMMON_SETUP,evince))

evince_OPTIONAL_DEPS=libgnome-keyring dbus-glib gconf nautilus libtiff ghostscript libspectre
evince_OPTIONAL_DEPS_TARGETS=$(foreach pkg,$(evince_OPTIONAL_DEPS), $(if $(TSWO_SOFTWARE_$(pkg)),$(pkg)-host-install,))
evince-patch: $(evince_OPTIONAL_DEPS_TARGETS)

ifneq (y,$(TSWO_SOFTWARE_libgnome-keyring))
evince_CONFIGURE_OPTIONS+=--without-keyring
endif
ifneq (y,$(TSWO_SOFTWARE_gconf))
evince_CONFIGURE_OPTIONS+=--without-gconf
endif
ifneq (y,$(TSWO_SOFTWARE_nautilus))
evince_CONFIGURE_OPTIONS+=--disable-nautilus
else
evince_CONFIGURE_OPTIONS+=nautilusextensiondir=/usr/lib/nautilus/extensions-2.0
endif
ifneq (y,$(TSWO_SOFTWARE_dbus-glib))
evince_CONFIGURE_OPTIONS+=--disable-dbus
endif
ifneq (y,$(TSWO_SOFTWARE_libtiff))
evince_CONFIGURE_OPTIONS+=--disable-tiff
endif

# Standard targets are defined as stamp files which are mapped to frontend
# targets.  For example, evince-configure maps to
# $(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_configured

# The following targets contain the commands for each step of the package
# build process.  In order to modify the commands for any step, uncomment
# the target and modify as required.  To discard the default commands for
# a particular target, uncomment and remove the body leaving only the
# target definition and the final @touch $@.

# evince-fetch is a frontend for:
#
#$(evince_SRC_DL_DIR)/$(evince_SOURCE):
#	$(call PRINT_INFO,fetching evince $(evince_VERSION))
#	@mkdir -p $(@D) $(evince_SRC_DL_DIR)
#	$(call FETCH_REMOTE,$(evince_SRC_DL_DIR),$(evince_SOURCE_URL))
#	$(foreach patch,$(evince_PATCHES),$(call FETCH_REMOTE,$(evince_SRC_DL_DIR),$(evince_URL)/$(patch)))
#	$(foreach file,$(evince_FILES),$(call FETCH_REMOTE,$(evince_SRC_DL_DIR),$(evince_URL)/$(file)))

# evince-unpack is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_unpacked:
#	$(call PRINT_INFO,unpacking evince $(evince_VERSION))
#	@mkdir -p $(@D)
#	@$(INFLATE$(suffix $(evince_SOURCE))) $(evince_SRC_DL_DIR)/$(evince_SOURCE) | \
#		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
#	@touch $@

# evince-patch is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_patched:
#	$(call PRINT_INFO,patching evince $(evince_VERSION))
#	$(foreach patch,$(evince_PATCHES),$(call APPLY_PATCH,$(evince_DDIR),$(evince_SRC_DL_DIR)/$(patch)))
#	@touch $@


#evince-configure is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_pre_configured:
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_configured:
#	$(call PRINT_INFO,configuring evince $(evince_VERSION))
#	$(call CONFIG_UPDATE,$(evince_DDIR))
#	cd $(evince_DDIR) && rm -rf config.cache 2>/dev/null && \
#		PATH=$(BUILD_PATH) \
#		$(TARGET_ALIASES) \
#		$(TARGET_BUILD_OPTIONS) \
#        $(evince_BUILD_OPTIONS) \
#		./configure \
#			--target=$(TSWO_FULL_TARGET_NAME) \
#			--host=$(TSWO_FULL_TARGET_NAME) \
#			--build=$(TSWO_BUILD_NAME) \
#        	--prefix=/usr \
#        	--exec-prefix=/usr \
#			--bindir=/usr/bin \
#			--sbindir=/usr/sbin \
#			--libdir=/usr/lib \
#			--libexecdir=/usr/lib \
#			--sysconfdir=/etc \
#			--datadir=/usr/share \
#			--localstatedir=/var \
#			--mandir=/usr/share/man \
#			--infodir=/usr/share/info \
#			--includedir=/usr/include \
#			$(TARGET_CONFIGURE_OPTIONS) \
#        	$(evince_CONFIGURE_OPTIONS)
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_post_configured:
#	@touch $@

# evince-build is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_pre_built:
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_built:
#	$(call PRINT_INFO,building evince $(evince_VERSION))
#	cd $(evince_DDIR) && \
#		PATH=$(BUILD_PATH) \
#		$(TARGET_ALIASES) \
#		$(TARGET_BUILD_OPTIONS) \
#        $(evince_BUILD_OPTIONS) \
#		$(evince_MAKE_CMD)
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_post_built:
#	@touch $@

# evince-rfs-install is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_pre_rfs_installed:
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_rfs_installed:
#	$(call PRINT_INFO,installing evince $(evince_VERSION) to the RFS)
#	@mkdir -p $(TSWO_RFS_INSTALL_DIR)
#	cd $(evince_DDIR) && \
#		PATH=$(BUILD_PATH) \
#		$(TARGET_ALIASES) \
#        $(evince_BUILD_OPTIONS) \
#			$(FAKEROOT_CMD) $(evince_MAKE_CMD) DESTDIR=$(TSWO_RFS_INSTALL_DIR) install
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_post_rfs_installed:
#	@touch $@

# evince-host-install is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_pre_host_installed:
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_host_installed:
#	$(call PRINT_INFO,installing evince $(evince_VERSION) to host)
#	@mkdir -p $(TSWO_HOST_INSTALL_DIR)
#	cd $(evince_DDIR) && \
#		PATH=$(BUILD_PATH) \
#		$(TARGET_ALIASES) \
#        $(evince_BUILD_OPTIONS) \
#		$(FAKEROOT_CMD) $(evince_MAKE_CMD) DESTDIR=$(TSWO_HOST_INSTALL_DIR) install
#	$(call FIX_LIBTOOL_ARCHIVES,$(TSWO_HOST_INSTALL_DIR))
#	$(call FIX_CONFIG_SCRIPTS,$(TSWO_HOST_INSTALL_DIR))
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_post_host_installed:
#	@touch $@

# evince-package is a frontend for:
#
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_pre_packaged:
#	$(call PRINT_INFO,packaging evince $(evince_VERSION))
#	@mkdir -p $(evince_PKG_INSTALL_DIR)
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_packaged:
#	cd $(evince_DDIR) && \
#		PATH=$(BUILD_PATH) \
#		$(TARGET_ALIASES) \
#        $(evince_BUILD_OPTIONS) \
#		$(FAKEROOT_CMD) $(evince_MAKE_CMD) DESTDIR=$(evince_PKG_INSTALL_DIR) install
#	@touch $@
#$(BUILD_DIR)/evince-$(evince_VERSION)/.stamp_post_packaged:
#	$(call CREATE_PACKAGE, evince,$(evince_PKG_INSTALL_DIR),$(evince_PKG_NAME),$(evince_VERSION),$(evince_BUILDNUM),$(evince_RUNTIME_DEPENDS))
#	@rm -rf $(evince_PKG_INSTALL_DIR)
#	@touch $@


# add to software list of packages to be built, packaged,
# and installed to the rfs (depending on the configuration)
ifeq (y,$(TSWO_SOFTWARE_evince))
SOFTWARE+=evince
endif


# vim:set noexpandtab filetype=make: 
