# Copyright (C) 2007-2010 Timesys Corporation
# boost rules

# Define common macros for boost from include/common.mk
# This must come after declaring any variables.
$(eval $(call COMMON_SETUP,boost))

$(BUILD_DIR)/boost-$(boost_VERSION)/.stamp_pre_configured:
	cd $(boost_DDIR) && \
		for util in gcc g++ ar ranlib as ld;do ln -s $(TARGET_CROSS_PREFIX)$$util $$util;done
	cd $(boost_DDIR) && \
		if [ ! -f configure ]; then ln -s bootstrap.sh configure;fi
	@touch $@

$(BUILD_DIR)/boost-$(boost_VERSION)/.stamp_configured:
	$(call PRINT_INFO,configuring boost $(boost_VERSION))
	cd $(boost_DDIR) && rm -rf config.cache 2>/dev/null && \
		PATH=./:$(BUILD_PATH) \
		$(TARGET_BUILD_OPTIONS) \
		$(boost_BUILD_OPTIONS) \
		./configure \
        	--prefix=/usr \
        	--exec-prefix=/usr \
			--libdir=/usr/lib \
			--includedir=/usr/include \
        	$(boost_CONFIGURE_OPTIONS)
	@touch $@

$(BUILD_DIR)/boost-$(boost_VERSION)/.stamp_built:
	$(call PRINT_INFO,building boost $(boost_VERSION))
	cd $(boost_DDIR) && \
		PATH=./:$(BUILD_PATH) \
		$(boost_MAKE_CMD)
	@touch $@

$(BUILD_DIR)/boost-$(boost_VERSION)/.stamp_rfs_installed:
	$(call PRINT_INFO,installing boost $(boost_VERSION) to the RFS)
	@mkdir -p $(TSWO_RFS_INSTALL_DIR)
	cd $(boost_DDIR) && \
		PATH=./:$(BUILD_PATH) \
		$(FAKEROOT_CMD) $(boost_MAKE_CMD) install PREFIX=$(TSWO_RFS_INSTALL_DIR)/usr EPREFIX=$(TSWO_RFS_INSTALL_DIR)/usr LIBDIR=$(TSWO_RFS_INSTALL_DIR)/usr/lib INCLUDEDIR=$(TSWO_RFS_INSTALL_DIR)/usr/include
	cd $(TSWO_RFS_INSTALL_DIR)/usr/include && if [ ! -d boost ]; then ln -sf boost*/boost;fi
	@touch $@

$(BUILD_DIR)/boost-$(boost_VERSION)/.stamp_host_installed:
	$(call PRINT_INFO,installing boost $(boost_VERSION) to host)
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)
	cd $(boost_DDIR) && \
		PATH=./:$(BUILD_PATH) \
		$(FAKEROOT_CMD) $(boost_MAKE_CMD) install PREFIX=$(TSWO_HOST_INSTALL_DIR)/usr EPREFIX=$(TSWO_HOST_INSTALL_DIR)/usr LIBDIR=$(TSWO_HOST_INSTALL_DIR)/usr/lib INCLUDEDIR=$(TSWO_HOST_INSTALL_DIR)/usr/include
	cd $(TSWO_HOST_INSTALL_DIR)/usr/include && if [ ! -d boost ]; then ln -sf boost*/boost;fi
	@touch $@

$(BUILD_DIR)/boost-$(boost_VERSION)/.stamp_packaged:
	@mkdir -p $(boost_PKG_INSTALL_DIR)/usr/{lib,include}
	cd $(boost_DDIR) && \
		PATH=./:$(BUILD_PATH) \
		$(TARGET_ALIASES) \
        $(boost_BUILD_OPTIONS) \
		$(FAKEROOT_CMD) $(boost_MAKE_CMD) install PREFIX=$(boost_PKG_INSTALL_DIR)/usr EPREFIX=$(boost_PKG_INSTALL_DIR)/usr LIBDIR=$(boost_PKG_INSTALL_DIR)/usr/lib INCLUDEDIR=$(boost_PKG_INSTALL_DIR)/usr/include
	cd $(boost_PKG_INSTALL_DIR)/usr/include && if [ ! -d boost ]; then ln -sf boost*/boost;fi
	@touch $@

# add to software list
ifeq (y,$(TSWO_SOFTWARE_boost))
SOFTWARE+=boost
endif


# vim:set noexpandtab filetype=make: 
