# Copyright (C) 2007-2010 Timesys Corporation

$(eval $(call COMMON_SETUP,ltp))

ifeq (y,$(strip $(TSWO_SOFTWARE_libaio)))
ltp-unpack: libaio-host-install
endif

ifeq (20081231,$(ltp_VERSION))
ltp_MAKE_CMD=make PREFIX=/usr
ltp_TESTCASE_DEST=/usr/ltp-$(ltp_VERSION)
endif

$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_pre_configured:
	-touch $(ltp_DDIR)/include/mk/config.mk
	cd $(ltp_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
		$(ltp_BUILD_OPTIONS) \
		$(ltp_MAKE_CMD) autoconf
	@touch $@   

ifneq (20081231,$(ltp_VERSION))
# The new version must change configure to override prefix
$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_configured:
	$(call PRINT_INFO,configuring ltp $(ltp_VERSION))
	cd $(ltp_DDIR) && rm -rf config.cache 2>/dev/null && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
		$(ltp_BUILD_OPTIONS) \
		./configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
			--prefix=/usr/ltp-$(ltp_VERSION) \
			$(TARGET_CONFIGURE_OPTIONS) \
			$(ltp_CONFIGURE_OPTIONS)
	@touch $@
endif

$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_rfs_installed: INSTALL_DIR=$(TSWO_RFS_INSTALL_DIR)
$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_packaged: INSTALL_DIR=$(ltp_PKG_INSTALL_DIR)

ifeq (20081231,$(ltp_VERSION))
# Legacy support, didn't have a full install target back then.
$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_rfs_installed $(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_packaged:
	$(call PRINT_INFO,installing ltp $(ltp_VERSION) to the RFS)
	@mkdir -p $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/testcases
	@mkdir -p $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/pan
	@mkdir -p $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/tools
	cd $(ltp_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(ltp_BUILD_OPTIONS) \
		$(FAKEROOT_CMD) $(ltp_MAKE_CMD) DESTDIR=$(INSTALL_DIR) install
	cp -af $(ltp_DDIR)/{ChangeLog,IDcheck.sh,runalltests.sh,runltp,runltplite.sh,runtest,ver_linux} $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)
	cp -af $(ltp_DDIR)/tools/{genhtml.pl,html_report_header.txt} $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/tools
	ln -sfn /usr/bin/{bump,pan,scanner} $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/pan
	cp -af $(ltp_DDIR)/testcases/bin $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/testcases/
	cp -af $(ltp_SRC_DL_DIR)/diffresults.sh $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)
	chmod +x $(INSTALL_DIR)/$(ltp_TESTCASE_DEST)/diffresults.sh
	@touch $@
else
$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_rfs_installed $(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_packaged:
	$(call PRINT_INFO,installing ltp $(ltp_VERSION) to the RFS)
	cd $(ltp_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(ltp_BUILD_OPTIONS) \
		$(FAKEROOT_CMD) $(ltp_MAKE_CMD) DESTDIR=$(INSTALL_DIR) install
		@touch $@
endif

$(BUILD_DIR)/ltp-$(ltp_VERSION)/.stamp_host_installed:
	$(call PRINT_INFO,installing ltp $(ltp_VERSION) to host)
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)
	for DIR in lib include m4; do \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(ltp_BUILD_OPTIONS) \
		$(FAKEROOT_CMD) $(ltp_MAKE_CMD) -C $(ltp_DDIR)/$${DIR} DESTDIR=$(TSWO_HOST_INSTALL_DIR) install; \
	done
	$(call FIX_LIBTOOL_ARCHIVES,$(TSWO_HOST_INSTALL_DIR))
	$(call FIX_CONFIG_SCRIPTS,$(TSWO_HOST_INSTALL_DIR))
	@touch $@


# add to software list of packages to be built, packaged,
# and installed to the rfs (depending on the configuration)
ifeq (y,$(TSWO_SOFTWARE_ltp))
SOFTWARE+=ltp
endif


# vim:set noexpandtab filetype=make: 
