# Copyright (C) 2007-2010 Timesys Corporation
# bluez rules

# Define common macros for bluez from include/common.mk
# This must come after declaring any variables.
$(eval $(call COMMON_SETUP,bluez))

bluez_OPTIONAL_DEPS=alsa-lib gst-plugins-base gstreamer libsndfile libusb ofono
bluez_OPTIONAL_DEPS_TARGETS=$(foreach pkg,$(bluez_OPTIONAL_DEPS), $(if $(TSWO_SOFTWARE_$(pkg)),$(pkg)-host-install,))
bluez-patch: $(bluez_OPTIONAL_DEPS_TARGETS)

# startup script requires which
ifneq (y,$(TSWO_SOFTWARE_busybox))
bluez-source: which-source
bluez-package: which-package
bluez-rfs-install: which-rfs-install
endif

ifeq (y,$(TSWO_mips))
bluez_CONFIGURE_OPTIONS+=--disable-pie
endif

$(BUILD_DIR)/bluez-$(bluez_VERSION)/.stamp_pre_configured:
	cd $(bluez_DDIR) && \
		find . -name ltmain.sh -exec $(SED) -i.orig -re '/add_dir="-L\$$libdir"/d' {} \;
	@touch $@

$(BUILD_DIR)/bluez-$(bluez_VERSION)/.stamp_rfs_installed:
	$(call PRINT_INFO,installing bluez $(bluez_VERSION) to the RFS)
	@mkdir -p $(TSWO_RFS_INSTALL_DIR)
	cd $(bluez_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
        $(bluez_BUILD_OPTIONS) \
			$(FAKEROOT_CMD) $(bluez_MAKE_CMD) DESTDIR=$(TSWO_RFS_INSTALL_DIR) install
	-if [ -e $(TSWO_RFS_INSTALL_DIR)/etc/init.d/bluetooth ]; then \
		mv -f $(TSWO_RFS_INSTALL_DIR)/etc/init.d/bluetooth $(TSWO_RFS_INSTALL_DIR)/etc/init.d/S25-bluetooth; \
	fi
	@touch $@

$(BUILD_DIR)/bluez-$(bluez_VERSION)/.stamp_packaged:
	cd $(bluez_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
        $(bluez_BUILD_OPTIONS) \
		$(FAKEROOT_CMD) $(bluez_MAKE_CMD) DESTDIR=$(bluez_PKG_INSTALL_DIR) install
	-if [ -e $(bluez_PKG_INSTALL_DIR)/etc/init.d/bluetooth ]; then \
		mv -f $(bluez_PKG_INSTALL_DIR)/etc/init.d/bluetooth $(bluez_PKG_INSTALL_DIR)/etc/init.d/S25-bluetooth; \
	fi
	@touch $@

$(BUILD_DIR)/bluez-$(bluez_VERSION)/.stamp_post_host_installed:
	@rm -rf $(TSWO_HOST_INSTALL_DIR)/usr/bin/{l2ping,sdptool,hidd,pand,hcitool,ciptool,rfcomm,dund,dfutool}
	@rm -rf $(TSWO_HOST_INSTALL_DIR)/usr/sbin/{hciattach,hciconfig,bccmd,bluetoothd,hid2hci}
	@touch $@


# add to software list of packages to be built, packaged,
# and installed to the rfs (depending on the configuration)
ifeq (y,$(TSWO_SOFTWARE_bluez))
SOFTWARE+=bluez
endif


# vim:set noexpandtab filetype=make: 
