# Copyright (C) 2007-2010 Timesys Corporation
# nss rules

# Define common macros for nss from include/common.mk
# This must come after declaring any variables.
$(eval $(call COMMON_SETUP,nss))

nss_OPTIONAL_DEPS=openssl
nss_OPTIONAL_DEPS_TARGETS=$(foreach pkg,$(nss_OPTIONAL_DEPS), $(if $(TSWO_SOFTWARE_$(pkg)),$(pkg)-host-install,))
nss-patch: $(nss_OPTIONAL_DEPS_TARGETS)


$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_configured:
	@touch $@

$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_pre_built:
	cd $(nss_DDIR) && \
		$(SED) -i -re '/ARCHFLAG.*=.*\-m[0-9][0-9]/d' ../coreconf/Linux.mk 
	@touch $@

nss_USE_64BIT=
ifeq (y,$(TSWO_64BIT))
nss_USE_64BIT=USE_64=1
endif

$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_built:
	$(call PRINT_INFO,building nss $(nss_VERSION))
	cd $(nss_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(nss_MAKE_CMD) OS_TEST=$(TSWO_ARCH) COMPILER_TAG=_$(TSWO_FULL_TARGET_NAME)-gcc BUILD_OPT=1 $(nss_USE_64BIT) NSS_USE_SYSTEM_SQLITE=1 build_coreconf
	cd $(nss_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(nss_MAKE_CMD) OS_TEST=$(TSWO_ARCH) CC=$(TSWO_FULL_TARGET_NAME)-gcc CXX=$(TSWO_FULL_TARGET_NAME)-g++ RANLIB=$(TSWO_FULL_TARGET_NAME)-ranlib OBJCOPY=$(TSWO_FULL_TARGET_NAME)-objcopy NM=$(TSWO_FULL_TARGET_NAME)-nm BUILD_OPT=1 $(nss_USE_64BIT) NSS_USE_SYSTEM_SQLITE=1 build_dbm
	cd $(nss_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(nss_MAKE_CMD) OS_TEST=$(TSWO_ARCH) CC=$(TSWO_FULL_TARGET_NAME)-gcc CXX=$(TSWO_FULL_TARGET_NAME)-g++ RANLIB=$(TSWO_FULL_TARGET_NAME)-ranlib OBJCOPY=$(TSWO_FULL_TARGET_NAME)-objcopy NM=$(TSWO_FULL_TARGET_NAME)-nm BUILD_OPT=1 $(nss_USE_64BIT) NSS_USE_SYSTEM_SQLITE=1 all
	@touch $@


$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_packaged: DESTDIR=$(nss_PKG_INSTALL_DIR)
$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_rfs_installed: DESTDIR=$(TSWO_RFS_INSTALL_DIR)
$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_packaged $(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_rfs_installed:
	$(call PRINT_INFO,installing nss $(nss_VERSION))
	@mkdir -p $(DESTDIR)/usr/{bin,lib,include}/nss
	cd $(nss_DDIR) && \
		cp -fH ../../dist/*.OBJ/lib/* $(DESTDIR)/usr/lib/nss/
	cd $(nss_DDIR) && \
		cp -fH ../../dist/*.OBJ/bin/* $(DESTDIR)/usr/bin/
	cd $(nss_DDIR) && \
		cp -fH ../../dist/public/nss/* $(DESTDIR)/usr/include/nss/
	cd $(DESTDIR)/usr/lib && \
		ln -sf nss/libnss3.so && \
		ln -sf nss/libnssutil3.so && \
		ln -sf nss/libssl3.so && \
		ln -sf nss/libsmime3.so 
	mkdir -p $(DESTDIR)/usr/lib/pkgconfig
	-cp $(nss_SRC_DL_DIR)/nss.pc $(DESTDIR)/usr/lib/pkgconfig/
	@touch $@

$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_host_installed: DESTDIR=$(TSWO_HOST_INSTALL_DIR)
$(BUILD_DIR)/nss-$(nss_VERSION)/.stamp_host_installed:
	$(call PRINT_INFO,installing nss $(nss_VERSION) to the host)
	@mkdir -p $(DESTDIR)/usr/{lib,include}/nss
	cd $(nss_DDIR) && \
		cp -fH ../../dist/*.OBJ/lib/* $(DESTDIR)/usr/lib/nss/
	cd $(nss_DDIR) && \
		cp -fH ../../dist/public/nss/* $(DESTDIR)/usr/include/nss/
	cd $(DESTDIR)/usr/lib && \
		ln -sf nss/libnss3.so && \
		ln -sf nss/libnssutil3.so && \
		ln -sf nss/libssl3.so && \
		ln -sf nss/libsmime3.so 
	mkdir -p $(DESTDIR)/usr/lib/pkgconfig
	-cp $(nss_SRC_DL_DIR)/nss.pc $(DESTDIR)/usr/lib/pkgconfig/
	@touch $@



# add to software list of packages to be built, packaged,
# and installed to the rfs (depending on the configuration)
ifeq (y,$(TSWO_SOFTWARE_nss))
SOFTWARE+=nss
endif


# vim:set noexpandtab filetype=make: 
