# Copyright (C) 2007-2009 Timesys Corporation
$(eval $(call COMMON_SETUP,binutils))

$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_configured:
	$(call PRINT_INFO,configuring binutils $(binutils_VERSION))
	cd $(@D)/binutils-$(binutils_VERSION) && rm -rf config.cache 2>/dev/null && \
		$(HOST_BUILD_OPTIONS) \
		./configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_HOST_NAME) \
			--build=$(TSWO_FULL_BUILD_NAME) \
        	--prefix=$(TSWO_HOST_INSTALL_DIR) \
        	--with-sysroot=$(TSWO_HOST_INSTALL_DIR) \
			--disable-nls \
			--disable-werror \
			$(HOST_CONFIGURE_OPTIONS) \
        	$(binutils_CONFIGURE_OPTIONS)
	@touch $@

$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_built:
	$(call PRINT_INFO,building binutils $(binutils_VERSION))
	cd $(binutils_DDIR) && \
		PATH=$(BUILD_PATH) \
		$(binutils_MAKE_CMD) \
		$(HOST_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS)
	@touch $@

# make the cross strip available in the path first
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_post_host_installed:
	@cd $(TSWO_HOST_INSTALL_DIR)/bin && ln -sf $(TSWO_FULL_TARGET_NAME)-strip strip
	@touch $@

$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_host_installed:
	$(call PRINT_INFO,installing binutils to the host)
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)
	cd $(@D)/binutils-$(binutils_VERSION) && \
		PATH=$(BUILD_PATH) \
		$(binutils_MAKE_CMD) install
	@touch $@

# target binutils
BINUTILS_RFS_BUILDDIR=$(BUILD_DIR)/binutils-$(binutils_VERSION)/binutils-rfs-build
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_rfs_built: $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_patched
	$(call PRINT_INFO,building binutils $(binutils_VERSION) for target)
	mkdir -p $(@D)/binutils_rfs_build
	$(INFLATE$(suffix $(binutils_SOURCE))) $(binutils_SRC_DL_DIR)/$(binutils_SOURCE) | \
		$(TAR) -C $(@D)/binutils_rfs_build $(TSWO_TAR_OPTIONS) -
	mv -f $(@D)/binutils_rfs_build/binutils-$(binutils_VERSION) $(BINUTILS_RFS_BUILDDIR)
	rm -rf $(@D)/binutils_rfs_build
	$(foreach patch,$(binutils_PATCHES),$(call APPLY_PATCH,$(BINUTILS_RFS_BUILDDIR),$(binutils_SRC_DL_DIR)/$(patch)))
	cd $(BINUTILS_RFS_BUILDDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		CFLAGS_FOR_BUILD= \
		./configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
        	--prefix=/usr \
        	--exec-prefix=/usr \
			--disable-nls \
			--disable-werror \
			$(TARGET_CONFIGURE_OPTIONS) \
        	$(binutils_CONFIGURE_OPTIONS)
	cd $(BINUTILS_RFS_BUILDDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		$(binutils_MAKE_CMD)
	@touch $@

$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_rfs_installed: $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_rfs_built
	$(call PRINT_INFO,installing binutils $(binutils_VERSION) to the RFS)
	cd $(BINUTILS_RFS_BUILDDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		$(binutils_MAKE_CMD) DESTDIR=$(TSWO_RFS_INSTALL_DIR) install
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_packaged: $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_rfs_built
	cd $(BINUTILS_RFS_BUILDDIR) && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
        $(binutils_BUILD_OPTIONS) \
		$(FAKEROOT_CMD) $(binutils_MAKE_CMD) DESTDIR=$(binutils_PKG_INSTALL_DIR) install
	@touch $@

TOOLCHAIN+=binutils

$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_configured:
	$(call PRINT_INFO,configuring libbfd $(binutils_VERSION))
	$(call CONFIG_UPDATE,$(binutils_DDIR)/bfd)
	cd $(binutils_DDIR)/bfd && rm -rf config.cache 2>/dev/null && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		./configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
        	--prefix=/usr \
        	--exec-prefix=/usr \
			--libdir=/usr/lib \
			--infodir=/usr/share/info \
			--includedir=/usr/include \
			--disable-nls \
			--disable-werror \
			$(TARGET_CONFIGURE_OPTIONS) \
        	$(binutils_CONFIGURE_OPTIONS)
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_built:
	$(call PRINT_INFO,building libbfd $(binutils_VERSION))
	cd $(binutils_DDIR)/bfd && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		$(binutils_MAKE_CMD)
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_host_installed:
	$(call PRINT_INFO,installing libbfd $(binutils_VERSION) to host)
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)
	cd $(binutils_DDIR)/bfd && \
		cp -f .libs/libbfd.* $(TSWO_HOST_INSTALL_DIR)/usr/lib/ && \
		cp -f ../include/{ansidecl,bfdlink,symcat}.h bfd.h $(TSWO_HOST_INSTALL_DIR)/usr/include
	$(call FIX_LIBTOOL_ARCHIVES,$(TSWO_HOST_INSTALL_DIR))
	$(call FIX_CONFIG_SCRIPTS,$(TSWO_HOST_INSTALL_DIR))
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_rfs_installed:
	$(call PRINT_INFO,installing libbfd $(binutils_VERSION) to the RFS)
	@mkdir -p $(TSWO_RFS_INSTALL_DIR)/usr/{lib,include}
	cd $(binutils_DDIR)/bfd && \
		cp -f .libs/libbfd.* $(TSWO_RFS_INSTALL_DIR)/usr/lib/ && \
		cp -f ../include/{ansidecl,bfdlink,symcat}.h bfd.h $(TSWO_RFS_INSTALL_DIR)/usr/include
	@touch $@
libbfd_PKG_INSTALL_DIR=$(PACKAGE_DIR)/libbfd_install
libbfd_BUILDNUM=1
libbfd_PKG_NAME=$(PACKAGE_DIR)/libbfd-$(binutils_VERSION)-$(TSWO_FULL_TARGET_NAME)-$(libbfd_BUILDNUM).tgz
libbfd_RUNTIME_DEPENDS=
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_packaged:
	$(call PRINT_INFO,packaging libbfd $(binutils_VERSION))
	@mkdir -p $(libbfd_PKG_INSTALL_DIR)/usr/{lib,include}
	cd $(binutils_DDIR)/bfd && \
		cp -f .libs/libbfd.* $(libbfd_PKG_INSTALL_DIR)/usr/lib/ && \
		cp -f ../include/{ansidecl,bfdlink,symcat}.h bfd.h $(libbfd_PKG_INSTALL_DIR)/usr/include
	$(call CREATE_PACKAGE, libbfd,$(libbfd_PKG_INSTALL_DIR),$(libbfd_PKG_NAME),$(binutils_VERSION),$(libbfd_BUILDNUM),$(libbfd_RUNTIME_DEPENDS))
	@rm -rf $(libbfd_PKG_INSTALL_DIR)
	@touch $@
LIBBFD_HOSTINSTALL_DIR=$(BUILD_DIR)/binutils-$(binutils_VERSION)/libbfd_host_build/binutils-$(binutils_VERSION)
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_host_built:
	$(call PRINT_INFO,building libbfd for the host)
	mkdir -p $(@D)/libbfd_host_build
	mkdir -p $(TSWO_FAKE_INSTALL_DIR)/{lib,include}
	$(INFLATE$(suffix $(binutils_SOURCE))) $(binutils_SRC_DL_DIR)/$(binutils_SOURCE) | \
		$(TAR) -C $(@D)/libbfd_host_build $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(binutils_PATCHES),$(call APPLY_PATCH,$(LIBBFD_HOSTINSTALL_DIR)/,$(binutils_SRC_DL_DIR)/$(patch)))
	cd $(LIBBFD_HOSTINSTALL_DIR)/bfd && \
		$(HOST_BUILD_OPTIONS) \
		./configure \
			--prefix=$(TSWO_FAKE_INSTALL_DIR) \
			--disable-nls \
			--disable-werror \
			$(HOST_CONFIGURE_OPTIONS)
	cd $(LIBBFD_HOSTINSTALL_DIR)/bfd && \
		$(MAKE)
	cd $(LIBBFD_HOSTINSTALL_DIR)/bfd && \
		cp -f .libs/libbfd.a $(TSWO_FAKE_INSTALL_DIR)/lib/ && \
		cp -f .libs/libbfd.la $(TSWO_FAKE_INSTALL_DIR)/lib/ && \
		cp -f ../include/{ansidecl,bfdlink,symcat}.h bfd.h $(TSWO_FAKE_INSTALL_DIR)/include
	@touch $@

libbfd-fetch: binutils-fetch
libbfd-unpack: binutils-unpack
libbfd-patch: binutils-patch
libbfd-configure: libbfd-patch $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_configured
libbfd-build: libbfd-configure $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_built
libbfd-host-install: libbfd-build $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_host_installed
libbfd-rfs-install: libbfd-build $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_rfs_installed
libbfd-package: libbfd-build $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_packaged
libbfd-clean:
libbfd-distclean:
libbfd-host-build: binutils-fetch $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libbfd_host_built

$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_configured:
	$(call PRINT_INFO,configuring libiberty $(binutils_VERSION))
	$(call CONFIG_UPDATE,$(binutils_DDIR)/libiberty)
	cd $(binutils_DDIR)/libiberty && rm -rf config.cache 2>/dev/null && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		./configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
        	--prefix=/usr \
        	--exec-prefix=/usr \
			--libdir=/usr/lib \
			--infodir=/usr/share/info \
			--includedir=/usr/include \
			--disable-nls \
			--disable-werror \
			$(TARGET_CONFIGURE_OPTIONS) \
        	$(binutils_CONFIGURE_OPTIONS)
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_built:
	$(call PRINT_INFO,building libiberty $(binutils_VERSION))
	cd $(binutils_DDIR)/libiberty && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
        $(binutils_BUILD_OPTIONS) \
		$(binutils_MAKE_CMD)
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_host_installed:
	@touch $@
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_rfs_installed:
	$(call PRINT_INFO,installing libiberty $(binutils_VERSION) to the RFS)
	mkdir -p $(TSWO_RFS_INSTALL_DIR)/usr/{lib,include}
	cd $(binutils_DDIR)/libiberty && \
		cp -f libiberty.a $(TSWO_RFS_INSTALL_DIR)/usr/lib/ && \
		cp -f ../include/libiberty.h $(TSWO_RFS_INSTALL_DIR)/usr/include/
	@touch $@
libiberty_PKG_INSTALL_DIR=$(PACKAGE_DIR)/libiberty_install
libiberty_BUILDNUM=1
libiberty_PKG_NAME=$(PACKAGE_DIR)/libiberty-$(binutils_VERSION)-$(TSWO_FULL_TARGET_NAME)-$(libiberty_BUILDNUM).tgz
libiberty_RUNTIME_DEPENDS=
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_packaged:
	$(call PRINT_INFO,packaging libiberty $(binutils_VERSION))
	@mkdir -p $(libiberty_PKG_INSTALL_DIR)/usr/{lib,include}
	cd $(binutils_DDIR)/libiberty && \
		cp -f libiberty.a $(libiberty_PKG_INSTALL_DIR)/usr/lib/ && \
		cp -f ../include/libiberty.h $(libiberty_PKG_INSTALL_DIR)/usr/include/
	$(call CREATE_PACKAGE, libiberty,$(libiberty_PKG_INSTALL_DIR),$(libiberty_PKG_NAME),$(binutils_VERSION),$(libiberty_BUILDNUM),$(libiberty_RUNTIME_DEPENDS))
	@rm -rf $(libiberty_PKG_INSTALL_DIR)
	@touch $@

LIBIBERTY_HOSTINSTALL_DIR=$(BUILD_DIR)/binutils-$(binutils_VERSION)/libiberty_host_build/binutils-$(binutils_VERSION)
$(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_host_built:
	$(call PRINT_INFO,building libiberty for the host)
	mkdir -p $(@D)/libiberty_host_build
	$(INFLATE$(suffix $(binutils_SOURCE))) $(binutils_SRC_DL_DIR)/$(binutils_SOURCE) | \
		$(TAR) -C $(@D)/libiberty_host_build $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(binutils_PATCHES),$(call APPLY_PATCH,$(LIBIBERTY_HOSTINSTALL_DIR)/,$(binutils_SRC_DL_DIR)/$(patch)))
	cd $(LIBIBERTY_HOSTINSTALL_DIR)/libiberty && \
		$(HOST_BUILD_OPTIONS) \
		./configure \
			--prefix=$(TSWO_FAKE_INSTALL_DIR) \
			$(HOST_CONFIGURE_OPTIONS)
	cd $(LIBIBERTY_HOSTINSTALL_DIR)/libiberty && \
		$(MAKE)
	cd $(LIBIBERTY_HOSTINSTALL_DIR)/libiberty && \
		cp -f libiberty.a $(TSWO_FAKE_INSTALL_DIR)/lib/ && \
		cp -f ../include/libiberty.h $(TSWO_FAKE_INSTALL_DIR)/include/
	@touch $@

libiberty-fetch: binutils-fetch
libiberty-unpack: binutils-unpack
libiberty-patch: binutils-patch
libiberty-configure: libiberty-patch $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_configured
libiberty-build: libiberty-configure $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_built
libiberty-host-install: libiberty-build $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_host_installed
libiberty-rfs-install: libiberty-build $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_rfs_installed
libiberty-package: libiberty-build $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_packaged
libiberty-clean:
libiberty-distclean:
libiberty-host-build: binutils-fetch $(BUILD_DIR)/binutils-$(binutils_VERSION)/.stamp_libiberty_host_built

# if the toolchain is to be added to the target
ifeq (y,$(strip $(TSWO_INSTALL_TOOLCHAIN_TO_TARGET)))
SOFTWARE+=binutils
endif
ifeq (y,$(strip $(TSWO_SOFTWARE_libbfd)))
SOFTWARE+=libbfd
endif
ifeq (y,$(strip $(TSWO_SOFTWARE_libiberty)))
SOFTWARE+=libiberty
endif

# vim:set noexpandtab: 

