# Copyright (C) 2007-2009 Timesys Corporation

# cygwin toolchain rules

CYGWIN_TOOLCHAIN_DIR=$(BUILD_DIR)/cygwin/toolchain

$(BUILD_DIR)/cygwin/.stamp_fetched:
	$(call PRINT_INFO,fetching cygwin files)
	@mkdir -p $(@D) $(CYGWIN_TOOLCHAIN_DIR)/{lib,include,bin}
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_unpacked:
	$(call PRINT_INFO,unpacking cygwin files)
	$(INFLATE$(suffix $($(COMPILER)_SOURCE))) $($(COMPILER)_SRC_DL_DIR)/$($(COMPILER)_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(INFLATE$(suffix $(binutils_SOURCE))) $(binutils_SRC_DL_DIR)/$(binutils_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(INFLATE$(suffix $(gmp_SOURCE))) $(gmp_SRC_DL_DIR)/$(gmp_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(INFLATE$(suffix $(mpfr_SOURCE))) $(mpfr_SRC_DL_DIR)/$(mpfr_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_patched:
	$(call PRINT_INFO,patching cygwin files)
	$(foreach patch,$(binutils_PATCHES),$(call APPLY_PATCH,$(@D)/binutils-$(binutils_VERSION),$(binutils_SRC_DL_DIR)/$(patch)))
	$(foreach patch,$($(COMPILER)_PATCHES),$(call APPLY_PATCH,$(@D)/$(COMPILER)-$($(COMPILER)_VERSION),$($(COMPILER)_SRC_DL_DIR)/$(patch)))
	$(foreach patch,$(gmp_PATCHES),$(call APPLY_PATCH,$(@D)/gmp-$(gmp_VERSION),$(gmp_SRC_DL_DIR)/$(patch)))
	$(foreach patch,$(mpfr_PATCHES),$(call APPLY_PATCH,$(@D)/mpfr-$(mpfr_VERSION),$(mpfr_SRC_DL_DIR)/$(patch)))
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_configured:
	$(call PRINT_INFO,configuring cygwin files)
	@cd $(CYGWIN_TOOLCHAIN_DIR) && ln -sf . usr
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_kernel_headers:
	$(call PRINT_INFO,building cygwin kernel headers)
	cd $(BUILD_DIR)/kernel-headers && PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) headers_install \
		ARCH=$(TSWO_KERNEL_HEADER_ARCH) CROSS_COMPILE=$(TSWO_FULL_TARGET_NAME)- \
		INSTALL_HDR_PATH=$(@D)/kernel_headers_inst
	cp -rf $(@D)/kernel_headers_inst/include/* $(CYGWIN_TOOLCHAIN_DIR)/include/
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_glibc:
	$(call PRINT_INFO,building cygwin glibc)
	cd $(glibc_DIR)/glibc-$(glibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(glibc_MAKE_CMD) install install_root=$(CYGWIN_TOOLCHAIN_DIR)
	@rm -f $(CYGWIN_TOOLCHAIN_DIR)/bin/{gencat,getconf,getent,locale,localedef,rpcgen,sprof,pcprofiledump}
	@rm -f $(CYGWIN_TOOLCHAIN_DIR)/sbin/{sln,zdump,zic,iconvconfig,nscd,ldconfig,rpcinfo}
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_uClibc:
	$(call PRINT_INFO,building cygwin uClibc)
	PATH=$(BUILD_PATH) \
		$(uClibc_MAKE_CMD) -C $(uClibc_DDIR) \
			PREFIX=$(CYGWIN_TOOLCHAIN_DIR) \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(CYGWIN_TOOLCHAIN_DIR)/include" \
			install_runtime install_dev
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_binutils:
	$(call PRINT_INFO,building cygwin binutils)
	rm -rf $(@D)/binutils-$(binutils_VERSION)-build
	mkdir -p $(@D)/binutils-$(binutils_VERSION)-build
	cd $(@D)/binutils-$(binutils_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../binutils-$(binutils_VERSION)/configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR) \
			--with-sysroot=$(CYGWIN_TOOLCHAIN_DIR) \
			--disable-nls --disable-werror
	cd $(@D)/binutils-$(binutils_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make all
	cd $(@D)/binutils-$(binutils_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_gmp:
	$(call PRINT_INFO,building cygwin gmp)
	rm -rf $(@D)/gmp-$(gmp_VERSION)-build
	mkdir -p $(@D)/gmp-$(gmp_VERSION)-build
	cd $(@D)/gmp-$(gmp_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../gmp-$(gmp_VERSION)/configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--prefix=$(@D)/gmp_inst
	cd $(@D)/gmp-$(gmp_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make
	cd $(@D)/gmp-$(gmp_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_mpfr:
	$(call PRINT_INFO,building cygwin mpfr)
	rm -rf $(@D)/mpfr-$(mpfr_VERSION)-build
	mkdir -p $(@D)/mpfr-$(mpfr_VERSION)-build
	cd $(@D)/mpfr-$(mpfr_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../mpfr-$(mpfr_VERSION)/configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--with-gmp=$(@D)/gmp_inst \
			--prefix=$(@D)/mpfr_inst
	cd $(@D)/mpfr-$(mpfr_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make
	cd $(@D)/mpfr-$(mpfr_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_built_gcc:
	$(call PRINT_INFO,building cygwin gcc)
	rm -rf $(@D)/gcc-$(gcc_VERSION)-build
	mkdir -p $(@D)/gcc-$(gcc_VERSION)-build
	cd $(@D)/gcc-$(gcc_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../gcc-$(gcc_VERSION)/configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR) \
			--with-sysroot=$(CYGWIN_TOOLCHAIN_DIR) \
			--disable-shared \
			--with-gnu-ld --with-gnu-as \
			--disable-libgcj --disable-libgomp --disable-libmudflap --disable-libssp --disable-nls \
			--enable-languages=$(TSWO_COMPILER_LANGUAGES) \
			--with-gmp=$(@D)/gmp_inst \
			--with-mpfr=$(@D)/mpfr_inst
	cd $(@D)/gcc-$(gcc_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make
	cd $(@D)/gcc-$(gcc_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_toolchain_xml:
	$(call PRINT_INFO,Generating cygwin toolchain.xml file)
	@echo "<toolchains>" > $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<toolchain name='GCC $(gcc_VERSION) $(TSWO_FULL_TARGET_NAME)-' id='$(TSWO_FULL_TARGET_NAME)-gcc-$(gcc_VERSION)' path='$(TSWO_HOST_INSTALL_DIR)/bin'>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<shortdesc>Toolchain for $(subst ",,$(TSWO_BOARD)) development.</shortdesc>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<longdesc>Toolchain for $(subst ",,$(TSWO_BOARD)) development.</longdesc>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tools>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='AR' path='' name='$(TSWO_FULL_TARGET_NAME)-ar.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='AS' path='' name='$(TSWO_FULL_TARGET_NAME)-as.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='CC' path='' name='$(TSWO_FULL_TARGET_NAME)-gcc.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
ifeq (y,$(TSWO_gcc_BUILD_CXX))
	@echo "<tool type='CXX' path='' name='$(TSWO_FULL_TARGET_NAME)-g++.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
endif
	@echo "<tool type='CPP' path='' name='$(TSWO_FULL_TARGET_NAME)-cpp.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='LD' path='' name='$(TSWO_FULL_TARGET_NAME)-ld.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='NM' path='' name='$(TSWO_FULL_TARGET_NAME)-nm.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='RANLIB' path='' name='$(TSWO_FULL_TARGET_NAME)-ranlib.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='OBJCOPY' path='' name='$(TSWO_FULL_TARGET_NAME)-objcopy.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='OBJDUMP' path='' name='$(TSWO_FULL_TARGET_NAME)-objdump.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "<tool type='STRIP' path='' name='$(TSWO_FULL_TARGET_NAME)-strip.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
ifeq (y,$(TSWO_GDB))
	@echo "<tool type='DEBUGGER' path='' name='$(TSWO_FULL_TARGET_NAME)-gdb.exe' />" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
endif
	@echo "</tools>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "</toolchain>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@echo "</toolchains>" >> $(CYGWIN_TOOLCHAIN_DIR)/toolchain.xml
	@touch $@

cygwin-installer: rfs-image-tar kernel-build kernel-source-package cygwin-package
	$(call PRINT_INFO,Creating cygwin distribution installer)
	mkdir -p $(BUILD_DIR)/installer
	cp -f $(BUILD_DIR)/toolchain-cygwin-*.tgz $(BUILD_DIR)/installer/toolchain.tgz
	cp -f $(IMAGES_DIR)/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION) $(BUILD_DIR)/installer/
	cp -f $(IMAGES_DIR)/rfs/rootfs.tar.gz $(BUILD_DIR)/installer/
	-cp -f $(IMAGES_DIR)/*.dtb $(BUILD_DIR)/installer/
	-cp -f $(PACKAGE_DIR)/kernel-source-*.tgz $(BUILD_DIR)/installer/kernel-source.tgz
	cp -f $(BASE_DIR)/target/installer/install $(BUILD_DIR)/installer/
	if [ -f $(BASE_DIR)/eula ]; then cp -f $(BASE_DIR)/eula $(BUILD_DIR)/installer/;fi
	chmod +x $(BUILD_DIR)/installer/install
	sed -i -r \
		-e "s/TSWO_BOARD/$(TSWO_BOARD)/" \
		-e "s/TSWO_ARCH/$(TSWO_ARCH)/" \
		-e "s,TSWO_HOST_INSTALL_DIR,$(CYGWIN_TOOLCHAIN_DIR)," \
		-e "s/TSWO_FULL_TARGET_NAME/$(TSWO_FULL_TARGET_NAME)/" \
		-e "s/TSWO_kernel_VERSION/$(kernel_VERSION)/" \
		-e "s/TSWO_KERNEL_FILENAME/$(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION)/" \
		$(BUILD_DIR)/installer/install
	cd $(BUILD_DIR)/installer && \
		$(TAR) --ignore-failed-read -czf installation.tar.gz eula install kernel-source.tgz rootfs.tar.gz toolchain.tgz $(FINAL_KERNEL_SHORTFILENAME)-$(kernel_VERSION)$(kernel_EXTRAVERSION)
	cd $(BUILD_DIR)/installer && \
		cat $(BASE_DIR)/target/installer/self-extract.sh > $(BUILD_DIR)/$(TSWO_BOARD)-cygwin-development-environment.sh
	cd $(BUILD_DIR)/installer && \
		cat installation.tar.gz >> $(BUILD_DIR)/$(TSWO_BOARD)-cygwin-development-environment.sh
	chmod +x $(BUILD_DIR)/$(TSWO_BOARD)-cygwin-development-environment.sh
	rm -rf $(BUILD_DIR)/installer

cygwin_toolchain_archive=$(BUILD_DIR)/toolchain-cygwin-$(TSWO_FULL_TARGET_NAME).tgz
$(cygwin_toolchain_archive):
	$(call PRINT_INFO,packaging cygwin toolchain)
	$(call CREATE_TGZ, $(CYGWIN_TOOLCHAIN_DIR),$(cygwin_toolchain_archive))

$(BUILD_DIR)/cygwin/.stamp_cramfs:
	$(call PRINT_INFO,installing cygwin cramfs)
	$(INFLATE$(suffix $(cramfs_SOURCE))) $(cramfs_SRC_DL_DIR)/$(cramfs_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(cramfs_PATCHES),$(call APPLY_PATCH,$(@D)/cramfs-$(cramfs_VERSION),$(cramfs_SRC_DL_DIR)/$(patch)))
	cd $(@D)/cramfs-$(cramfs_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(cramfs_MAKE_CMD) clean all \
		$(CYGWINX_ALIASES)
	cd $(@D)/cramfs-$(cramfs_VERSION) && \
		cp cramfsck $(CYGWIN_TOOLCHAIN_DIR)/bin/cramfsck.exe && \
		cp mkcramfs $(CYGWIN_TOOLCHAIN_DIR)/bin/mkcramfs.exe
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_gdb:
	$(call PRINT_INFO,installing cygwin gdb)
	$(INFLATE$(suffix $(gdb_SOURCE))) $(gdb_SRC_DL_DIR)/$(gdb_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(gdb_PATCHES),$(call APPLY_PATCH,$(@D)/gdb-$(gdb_VERSION),$(gdb_SRC_DL_DIR)/$(patch)))
	rm -rf $(@D)/gdb-$(gdb_VERSION)-build
	mkdir -p $(@D)/gdb-$(gdb_VERSION)-build
	cd $(@D)/gdb-$(gdb_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../gdb-$(gdb_VERSION)/configure \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR) \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--with-gmp=$(@D)/gmp_inst \
            --with-sysroot=$(CYGWIN_TOOLCHAIN_DIR)/. \
			--disable-nls \
			$(gdb_CONFIGURE_OPTIONS)
	cd $(@D)/gdb-$(gdb_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE)
	cd $(@D)/gdb-$(gdb_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_genext2fs:
	$(call PRINT_INFO,installing cygwin genext2fs)
	$(INFLATE$(suffix $(genext2fs_SOURCE))) $(genext2fs_SRC_DL_DIR)/$(genext2fs_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(genext2fs_PATCHES),$(call APPLY_PATCH,$(@D)/genext2fs-$(genext2fs_VERSION),$(genext2fs_SRC_DL_DIR)/$(patch)))
	cd $(@D)/genext2fs-$(genext2fs_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		./configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(cygwin_TARGET) \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR)/ \
        	$(genext2fs_CONFIGURE_OPTIONS)
	cd $(@D)/genext2fs-$(genext2fs_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(genext2fs_MAKE_CMD)
	cd $(@D)/genext2fs-$(genext2fs_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(genext2fs_MAKE_CMD) install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_intltool:
	$(call PRINT_INFO,installing cygwin intltool)
	$(INFLATE$(suffix $(intltool_SOURCE))) $(intltool_SRC_DL_DIR)/$(intltool_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(intltool_PATCHES),$(call APPLY_PATCH,$(@D)/intltool-$(intltool_VERSION),$(intltool_SRC_DL_DIR)/$(patch)))
	cd $(@D)/intltool-$(intltool_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		./configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(cygwin_TARGET) \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR)/ \
        	$(intltool_CONFIGURE_OPTIONS)
	cd $(@D)/intltool-$(intltool_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(intltool_MAKE_CMD)
	cd $(@D)/intltool-$(intltool_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(intltool_MAKE_CMD) install
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_mtd-utils:
	$(call PRINT_INFO,installing cygwin mtd-utils)
	$(INFLATE$(suffix $(mtd-utils_SOURCE))) $(mtd-utils_SRC_DL_DIR)/$(mtd-utils_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(mtd-utils_PATCHES),$(call APPLY_PATCH,$(@D)/mtd-utils-$(mtd-utils_VERSION),$(mtd-utils_SRC_DL_DIR)/$(patch)))
	cd $(@D)/mtd-utils-$(mtd-utils_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
			$(mtd-utils_MAKE_CMD) \
			CC="$(cygwin_TARGET)-gcc -D__USE_LINUX_IOCTL_DEFS=1 -D'IFTODT(__mode)'='(((__mode) & 0170000) >> 12)'"
	cd $(@D)/mtd-utils-$(mtd-utils_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
			$(mtd-utils_MAKE_CMD) DESTDIR=$(CYGWIN_TOOLCHAIN_DIR) install \
			CC="$(cygwin_TARGET)-gcc -D__USE_LINUX_IOCTL_DEFS=1 -D'IFTODT(__mode)'='(((__mode) & 0170000) >> 12)'"
	@touch $@

# XXX this needs glib etc to build... not sure it is worth it
$(BUILD_DIR)/cygwin/.stamp_pkgconfig:
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_ccache:
	$(call PRINT_INFO,installing cygwin ccache)
	$(INFLATE$(suffix $(ccache_SOURCE))) $(ccache_SRC_DL_DIR)/$(ccache_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(ccache_PATCHES),$(call APPLY_PATCH,$(@D)/ccache-$(ccache_VERSION),$(ccache_SRC_DL_DIR)/$(patch)))
	cd $(@D)/ccache-$(ccache_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		./configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(cygwin_TARGET) \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR)/ \
        	$(ccache_CONFIGURE_OPTIONS)
	cd $(@D)/ccache-$(ccache_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(ccache_MAKE_CMD)
	cd $(@D)/ccache-$(ccache_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(ccache_MAKE_CMD) install
	cd $(CYGWIN_TOOLCHAIN_DIR) && \
		mkdir -p ccache && \
		cd ccache && \
		if [ -x ../bin/$(TSWO_FULL_TARGET_NAME)-gcc.exe ]; then ln -sf ../bin/ccache $(TSWO_FULL_TARGET_NAME)-gcc.exe;fi && \
		if [ -x ../bin/$(TSWO_FULL_TARGET_NAME)-g++.exe ]; then ln -sf ../bin/ccache $(TSWO_FULL_TARGET_NAME)-g++.exe;fi && \
		ln -sf ../bin/ccache gcc.exe && \
		ln -sf ../bin/ccache g++.exe && \
		ln -sf ../bin/ccache $(cygwin_TARGET)-gcc.exe && \
		ln -sf ../bin/ccache $(cygwin_TARGET)-g++.exe
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_squashfs:
	$(call PRINT_INFO,installing cygwin squashfs)
	$(INFLATE$(suffix $(squashfs_SOURCE))) $(squashfs_SRC_DL_DIR)/$(squashfs_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(squashfs_PATCHES),$(call APPLY_PATCH,$(@D)/squashfs$(squashfs_VERSION),$(squashfs_SRC_DL_DIR)/$(patch)))
	cd $(@D)/squashfs$(squashfs_VERSION)/squashfs-tools && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(squashfs_MAKE_CMD) \
		$(CYGWINX_ALIASES)
	cp $(@D)/squashfs$(squashfs_VERSION)/squashfs-tools/{mksquashfs,unsquashfs} $(CYGWIN_TOOLCHAIN_DIR)/bin
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_tsrpm:
	$(call PRINT_INFO,installing cygwin tsrpm)
	$(INFLATE$(suffix $(tsrpm_SOURCE))) $(tsrpm_SRC_DL_DIR)/$(tsrpm_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(tsrpm_PATCHES),$(call APPLY_PATCH,$(@D)/tsrpm$(tsrpm_VERSION),$(tsrpm_SRC_DL_DIR)/$(patch)))
	cd $(@D)/tsrpm-$(tsrpm_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
        $(tsrpm_BUILD_OPTIONS) \
		$(tsrpm_MAKE_CMD) files= rpm_files= TARGET=cygwin \
			$(CYGWINX_ALIASES)
	cd $(@D)/tsrpm-$(tsrpm_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
        $(tsrpm_BUILD_OPTIONS) \
			$(FAKEROOT_CMD) $(tsrpm_MAKE_CMD) prefix=$(CYGWIN_TOOLCHAIN_DIR) install \
				files= rpm_files= man1dir=$(CYGWIN_TOOLCHAIN_DIR)/usr/share/man/man1 \
				man5dir=$(CYGWIN_TOOLCHAIN_DIR)/usr/share/man/man5 TARGET=cygwin
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_mkimage:
	$(call PRINT_INFO,building cygwin mkimage)
	$(INFLATE$(suffix $(u-boot_SOURCE))) $(u-boot_SRC_DL_DIR)/$(u-boot_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(u-boot_PATCHES),$(call APPLY_PATCH,$(@D)/u-boot-$(u-boot_VERSION),$(u-boot_SRC_DL_DIR)/$(patch)))
	cd $(@D)/u-boot-$(u-boot_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(cygwin_TARGET)-gcc -DUSE_HOSTCC -Iinclude/ -o $(CYGWIN_TOOLCHAIN_DIR)/bin/mkimage.exe tools/mkimage.c $$(find . -type f -name crc32.c)
	cd $(@D)/u-boot-$(u-boot_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(cygwin_TARGET)-gcc -DUSE_HOSTCC -o $(CYGWIN_TOOLCHAIN_DIR)/bin/img2srec.exe tools/img2srec.c
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_opxml:
	$(call PRINT_INFO,building cygwin opxml)
	$(INFLATE$(suffix $(opxml_SOURCE))) $(opxml_SRC_DL_DIR)/$(opxml_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(opxml_PATCHES),$(call APPLY_PATCH,$(@D)/opxml-$(opxml_VERSION),$(opxml_SRC_DL_DIR)/$(patch)))
	cd $(@D)/opxml-$(opxml_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(opxml_MAKE_CMD) \
			$(CYGWINX_ALIASES) \
			CFLAGS="-I$(@D)/oprofile-$(oprofile_VERSION)/{libdb,libop,libutil,libutil++} -fno-use-cxa-atexit" \
			LDFLAGS="-L$(@D)/oprofile-$(oprofile_VERSION)/{libdb,libop,libutil,libutil++}" \
			OPROFILE_LIBS="-lop -lodb -lutil++ -lutil"
	cd $(@D)/opxml-$(opxml_VERSION) && \
		cp -f opxml $(CYGWIN_TOOLCHAIN_DIR)/bin/opxml.exe
	@touch $@

$(BUILD_DIR)/cygwin/.stamp_opimport:
	$(call PRINT_INFO,building opimport for cygwin)
	$(INFLATE$(suffix $(oprofile_SOURCE))) $(oprofile_SRC_DL_DIR)/$(oprofile_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(oprofile_PATCHES),$(call APPLY_PATCH,$(@D)/oprofile-$(oprofile_VERSION),$(oprofile_SRC_DL_DIR)/$(patch)))
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		CXXFLAGS=-fno-use-cxa-atexit \
		$(CYGWINX_ALIASES) \
		./configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(cygwin_TARGET) \
			--prefix=$(CYGWIN_TOOLCHAIN_DIR)/ \
        	$(oprofile_CONFIGURE_OPTIONS)
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) -C libdb libodb.a
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) -C libopt++ libopt++.a
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) -C libutil++ libutil++.a
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) -C libutil libutil.a
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) -C libop libop.a
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE) -C libabi opimport.exe
	cd $(@D)/oprofile-$(oprofile_VERSION) && \
		cp -f libabi/opimport.exe $(CYGWIN_TOOLCHAIN_DIR)/bin/opimport.exe
	@touch $@



.PHONY: cygwin-fetch cygwin-unpack cygwin-patch cygwin-configure cygwin-build-kernel_headers cygwin-build-glibc cygwin-build-uClibc cygwin-build-binutils cygwin-build-gmp cygwin-build-mpfr cygwin-build-gcc cygwin-build cygwin-package cygwin cramfs-cygwin-install gdb-cygwin-install genext2fs-cygwin-install intltool-cygwin-install mtd-utils-cygwin-install pkgconfig-cygwin-install squashfs-cygwin-install tsrpm-cygwin-install mkimage-cygwin-install ccache-cygwin-install opxml-cygwin-install opimport-cygwin-install

CYGWIN_UTILITIES_INSTALL=
ifeq (y,$(TSWO_GDB))
CYGWIN_UTILITIES_INSTALL+=gdb-cygwin-install
endif
# XXX these probably do not work
#ifeq (y, $(strip $(TSWO_HOSTTOOL_cramfs)))
#CYGWIN_UTILITIES_INSTALL+=cramfs-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_genext2fs)))
#CYGWIN_UTILITIES_INSTALL+=genext2fs-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_intltool)))
#CYGWIN_UTILITIES_INSTALL+=intltool-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_mtd-utils)))
#CYGWIN_UTILITIES_INSTALL+=mtd-utils-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_pkgconfig)))
#CYGWIN_UTILITIES_INSTALL+=pkgconfig-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_squashfs)))
#CYGWIN_UTILITIES_INSTALL+=squashfs-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_ccache)))
#CYGWIN_UTILITIES_INSTALL+=ccache-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_tsrpm)))
#CYGWIN_UTILITIES_INSTALL+=tsrpm-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_mkimage)))
#CYGWIN_UTILITIES_INSTALL+=mkimage-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_opimport)))
#CYGWIN_UTILITIES_INSTALL+=opimport-cygwin-install
#endif
#ifeq (y, $(strip $(TSWO_HOSTTOOL_opxml)))
#CYGWIN_UTILITIES_INSTALL+=opxml-cygwin-install
#endif

cygwin-fetch: cygwinx-fetch gmp-fetch mpfr-fetch $(BUILD_DIR)/cygwin/.stamp_fetched
cygwin-unpack: cygwin-fetch $(BUILD_DIR)/cygwin/.stamp_unpacked
cygwin-patch: cygwin-unpack $(BUILD_DIR)/cygwin/.stamp_patched
cygwin-configure: cygwin-patch $(BUILD_DIR)/cygwin/.stamp_configured
cygwin-build-kernel_headers: host-kernel-headers $(BUILD_DIR)/cygwin/.stamp_built_kernel_headers
cygwin-build-glibc: cygwinx-host-install glibc-host-install $(BUILD_DIR)/cygwin/.stamp_built_glibc
cygwin-build-uClibc: cygwinx-host-install uClibc-host-install $(BUILD_DIR)/cygwin/.stamp_built_uClibc
cygwin-build-binutils: cygwinx-host-install $(BUILD_DIR)/cygwin/.stamp_built_binutils
cygwin-build-gmp: $(BUILD_DIR)/cygwin/.stamp_built_gmp
cygwin-build-mpfr: cygwin-build-gmp $(BUILD_DIR)/cygwin/.stamp_built_mpfr
cygwin-build-gcc: cygwin-build-mpfr $(BUILD_DIR)/cygwin/.stamp_built_gcc
cramfs-cygwin-install: cygwinx-zlib cramfs-fetch cygwin-configure cygwinx-build $(BUILD_DIR)/cygwin/.stamp_cramfs
gdb-cygwin-install: cygwinx-ncurses gdb-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_gdb
genext2fs-cygwin-install: genext2fs-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_genext2fs
intltool-cygwin-install: intltool-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_intltool
mtd-utils-cygwin-install: zlib-host-install mtd-utils-fetch cygwinx-build cygwin-configure cygwinx-zlib $(BUILD_DIR)/cygwin/.stamp_mtd-utils
pkgconfig-cygwin-install: pkgconfig-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_pkgconfig
squashfs-cygwin-install: squashfs-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_squashfs
tsrpm-cygwin-install: tsrpm-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_tsrpm
ccache-cygwin-install: ccache-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_ccache
mkimage-cygwin-install: u-boot-fetch cygwinx-build cygwin-configure $(BUILD_DIR)/cygwin/.stamp_mkimage
opxml-cygwin-install: opxml-fetch opimport-cygwin-install $(BUILD_DIR)/cygwin/.stamp_opxml
opimport-cygwin-install: cygwinx-bfd cygwinx-popt oprofile-fetch $(BUILD_DIR)/cygwin/.stamp_opimport
cygwin-build: stage3_cc cygwinx-host-install cygwin-configure cygwin-build-kernel_headers cygwin-build-$(LIBC) cygwin-build-binutils cygwin-build-gmp cygwin-build-mpfr cygwin-build-$(COMPILER) $(CYGWIN_UTILITIES_INSTALL)
cygwin-toolchain_xml: cygwin-build $(BUILD_DIR)/cygwin/.stamp_toolchain_xml
cygwin-package: cygwin-build cygwin-toolchain_xml $(cygwin_toolchain_archive)

cygwin: cygwin-package


# vim:set noexpandtab filetype=make: 
