# Copyright (C) 2007-2009 Timesys Corporation

# cygwin cross compiler rules

cygwinx_SRC_DL_DIR=$(SRC_DL_DIR)/cygwinx
cygwinx_DIR=$(BUILD_DIR)/cygwinx
cygwin_TARGET=$(strip $(call UNQUOTE_VAR,$(TSWO_cygwin_TARGET)))
cygwin_binary_SOURCE=$(strip $(call UNQUOTE_VAR,$(TSWO_cygwin_binary_SOURCE)))
w32api_SOURCE=$(strip $(call UNQUOTE_VAR,$(TSWO_w32api_SOURCE)))
CYGWINX_BUILD_PATH=$(BUILD_DIR)/cygwinx/bin:$(BUILD_DIR)/cygwinx/usr/bin:$(BUILD_DIR)/cygwinx/sbin:$(BUILD_DIR)/cygwinx/usr/sbin:$(BUILD_PATH)
CYGWINX_ALIASES=CC=$(cygwin_TARGET)-gcc AR=$(cygwin_TARGET)-ar LD=$(cygwin_TARGET)-ld NM=$(cygwin_TARGET)-nm AS=$(cygwin_TARGET)-as RANLIB=$(cygwin_TARGET)-ranlib STRIP=$(cygwin_TARGET)-strip CXX=$(cygwin_TARGET)-g++ PKG_CONFIG=$(TSWO_HOST_INSTALL_DIR)/bin/pkg-config


$(BUILD_DIR)/cygwinx/.stamp_fetched:
	$(call PRINT_INFO,fetching cygwinx files)
	mkdir -p $(@D)/{lib,include} $(cygwinx_SRC_DL_DIR)
	$(call FETCH_REMOTE,$(cygwinx_SRC_DL_DIR),$(TSWO_cygwin_binary_URL)/$(cygwin_binary_SOURCE))
	$(call FETCH_REMOTE,$(cygwinx_SRC_DL_DIR),$(TSWO_w32api_URL)/$(w32api_SOURCE))
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_unpacked:
	$(call PRINT_INFO,unpacking cygwinx files)
	$(INFLATE$(suffix $($(COMPILER)_SOURCE))) $($(COMPILER)_SRC_DL_DIR)/$($(COMPILER)_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(INFLATE$(suffix $(binutils_SOURCE))) $(binutils_SRC_DL_DIR)/$(binutils_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(INFLATE$(suffix $(cygwin_binary_SOURCE))) $(cygwinx_SRC_DL_DIR)/$(cygwin_binary_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) - usr
	cd $(@D) && mv usr $(cygwin_TARGET)
	$(INFLATE$(suffix $(w32api_SOURCE))) $(cygwinx_SRC_DL_DIR)/$(w32api_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	cd $(@D) && mv include $(cygwin_TARGET)/include/w32api
	cd $(@D) && mv lib $(cygwin_TARGET)/lib/w32api
	mkdir -p $(@D)/{lib,include}
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_patched:
	$(call PRINT_INFO,patching cygwinx files)
	$(foreach patch,$(binutils_PATCHES),$(call APPLY_PATCH,$(@D)/binutils-$(binutils_VERSION),$(binutils_SRC_DL_DIR)/$(patch)))
	$(foreach patch,$($(COMPILER)_PATCHES),$(call APPLY_PATCH,$(@D)/$(COMPILER)-$($(COMPILER)_VERSION),$($(COMPILER)_SRC_DL_DIR)/$(patch)))
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_configured:
	$(call PRINT_INFO,configuring cygwinx files)
	@cd $(@D) && ln -s . usr
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_built_binutils:
	$(call PRINT_INFO,building cygwinx binutils)
	rm -rf $(@D)/binutils-$(binutils_VERSION)-build
	mkdir -p $(@D)/binutils-$(binutils_VERSION)-build
	cd $(@D)/binutils-$(binutils_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../binutils-$(binutils_VERSION)/configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(TSWO_HOST_NAME) \
			--target=$(cygwin_TARGET) \
			--prefix=$(@D) \
			--disable-werror \
			--with-sysroot=$(@D)
	cd $(@D)/binutils-$(binutils_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make all
	cd $(@D)/binutils-$(binutils_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make install
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_built_gcc:
	$(call PRINT_INFO,building cygwinx gcc)
	rm -rf $(@D)/gcc-$(gcc_VERSION)-build
	mkdir -p $(@D)/gcc-$(gcc_VERSION)-build
	cd $(@D)/gcc-$(gcc_VERSION)-build && \
		$(SED) -i -re '/No support for this host/{n;n; s/\{ \(exit 1\); exit 1; \};//}' ../gcc-$(gcc_VERSION)/libstdc*/configure && \
		PATH=$(CYGWINX_BUILD_PATH) \
		../gcc-$(gcc_VERSION)/configure \
			--build=$(TSWO_HOST_NAME) \
			--host=$(TSWO_HOST_NAME) \
			--target=$(cygwin_TARGET) \
			--prefix=$(@D) \
			--with-sysroot=$(@D) \
			--enable-languages=$(TSWO_COMPILER_LANGUAGES) \
			--with-gmp=$(GMP_HOST_INSTALL_DIR) \
			--with-mpfr=$(MPFR_HOST_INSTALL_DIR) \
			--with-gnu-ld --with-gnu-as --with-system-zlib --disable-libssp --disable-libmudflap --disable-libgomp
	cd $(@D)/gcc-$(gcc_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make 
	cd $(@D)/gcc-$(gcc_VERSION)-build && \
		PATH=$(CYGWINX_BUILD_PATH) \
		make install
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_built:
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_zlib:
	$(call PRINT_INFO,building cygwinx zlib)
	$(INFLATE$(suffix $(zlib_SOURCE))) $(zlib_SRC_DL_DIR)/$(zlib_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(zlib_PATCHES),$(call APPLY_PATCH,$(@D)/zlib-$(zlib_VERSION),$(zlib_SRC_DL_DIR)/$(patch)))
	cd $(@D)/zlib-$(zlib_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		AR='$(cygwin_TARGET)-ar rc' \
		./configure \
			--prefix=$(cygwinx_DIR)
	cd $(@D)/zlib-$(zlib_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		AR='$(cygwin_TARGET)-ar rc' \
		$(MAKE)
	cd $(@D)/zlib-$(zlib_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		AR='$(cygwin_TARGET)-ar rc' \
		$(MAKE) install
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_ncurses:
	$(call PRINT_INFO,building cygwinx ncurses)
	$(INFLATE$(suffix $(ncurses_SOURCE))) $(ncurses_SRC_DL_DIR)/$(ncurses_SOURCE) | \
	$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(ncurses_PATCHES),$(call APPLY_PATCH,$(@D)/ncurses-$(ncurses_VERSION),$(ncurses_SRC_DL_DIR)/$(patch)))
	cd $(@D)/ncurses-$(ncurses_VERSION) && \
		./configure && make && cp -rf progs progs.native && make distclean
	cd $(@D)/ncurses-$(ncurses_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(ncurses_BUILD_OPTIONS) \
		$(CYGWINX_ALIASES) \
		./configure \
			--target=$(cygwin_TARGET) \
			--host=$(TSWO_BUILD_NAME) \
			--prefix=/usr \
			--disable-shared \
			$(ncurses_CONFIGURE_OPTIONS)
	cd $(@D)/ncurses-$(ncurses_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		$(MAKE)
	cd $(@D)/ncurses-$(ncurses_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		$(MAKE) DESTDIR=$(cygwinx_DIR) install
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_popt:
	$(call PRINT_INFO,building cygwinx popt)
	$(INFLATE$(suffix $(popt_SOURCE))) $(popt_SRC_DL_DIR)/$(popt_SOURCE) | \
		$(TAR) -C $(@D) $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(popt_PATCHES),$(call APPLY_PATCH,$(@D)/popt-$(popt_VERSION),$(popt_SRC_DL_DIR)/$(patch)))
	cd $(@D)/popt-$(popt_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(popt_BUILD_OPTIONS) \
		$(CYGWINX_ALIASES) \
		./configure \
			--target=$(cygwin_TARGET) \
			--host=$(cygwin_TARGET) \
			--build=$(TSWO_BUILD_NAME) \
        	--prefix=/usr \
			--disable-shared \
			$(popt_CONFIGURE_OPTIONS)
	cd $(@D)/popt-$(popt_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		$(MAKE)
	cd $(@D)/popt-$(popt_VERSION) && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		$(MAKE) DESTDIR=$(cygwinx_DIR) install
	@touch $@

$(BUILD_DIR)/cygwinx/.stamp_bfd:
	$(call PRINT_INFO,building cygwinx bfd)
	mkdir $(@D)/libbfd
	$(INFLATE$(suffix $(binutils_SOURCE))) $(binutils_SRC_DL_DIR)/$(binutils_SOURCE) | \
		$(TAR) -C $(@D)/libbfd $(TSWO_TAR_OPTIONS) -
	$(foreach patch,$(binutils_PATCHES),$(call APPLY_PATCH,$(@D)/libbfd/binutils-$(binutils_VERSION),$(binutils_SRC_DL_DIR)/$(patch)))
	cd $(@D)/libbfd/binutils-$(binutils_VERSION)/bfd && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(CYGWINX_ALIASES) \
		./configure \
			--build=$(TSWO_BUILD_NAME) \
			--host=$(cygwin_TARGET) \
			--target=$(cygwin_TARGET) \
        	--prefix=$(cygwinx_DIR) \
			--disable-nls
	cd $(@D)/libbfd/binutils-$(binutils_VERSION)/bfd && \
		PATH=$(CYGWINX_BUILD_PATH) \
		$(MAKE)
	cd $(@D)/libbfd/binutils-$(binutils_VERSION)/bfd && \
		cp -f .libs/libbfd.a $(cygwinx_DIR)/lib/ && \
		cp -f ../include/{ansidecl,bfdlink,symcat}.h bfd.h $(cygwinx_DIR)/include
	@touch $@


.PHONY: cygwinx-fetch cygwinx-unpack cygwinx-patch cygwinx-configure cygwinx-build cygwinx-host-install cygwinx cygwinx-zlib
cygwinx-fetch: binutils-fetch cc-fetch gmp-fetch mpfr-fetch $(BUILD_DIR)/cygwinx/.stamp_fetched
cygwinx-unpack: cygwinx-fetch $(BUILD_DIR)/cygwinx/.stamp_unpacked
cygwinx-patch: cygwinx-unpack $(BUILD_DIR)/cygwinx/.stamp_patched
cygwinx-configure: cygwinx-patch $(BUILD_DIR)/cygwinx/.stamp_configured
cygwinx-build-binutils: cygwinx-configure $(BUILD_DIR)/cygwinx/.stamp_built_binutils
cygwinx-build-$(COMPILER): gmp-toolchain-host-install mpfr-toolchain-host-install cygwinx-build-binutils $(BUILD_DIR)/cygwinx/.stamp_built_$(COMPILER)
cygwinx-build: cygwinx-build-$(COMPILER) $(BUILD_DIR)/cygwinx/.stamp_built
cygwinx-zlib: zlib-fetch cygwinx-build $(BUILD_DIR)/cygwinx/.stamp_zlib
cygwinx-ncurses: ncurses-fetch cygwinx-build $(BUILD_DIR)/cygwinx/.stamp_ncurses
cygwinx-popt: popt-fetch cygwinx-build $(BUILD_DIR)/cygwinx/.stamp_popt
cygwinx-bfd: binutils-fetch cygwinx-build $(BUILD_DIR)/cygwinx/.stamp_bfd
cygwinx-host-install: cygwinx-build

cygwinx: cygwinx-host-install


# vim:set noexpandtab filetype=make: 
