# Copyright (C) 2007-2009 Timesys Corporation
# vim:set noexpandtab: 

$(eval $(call COMMON_SETUP,eglibc))

ifeq ($(TSWO_EGLIBC_PORTS),y)
eglibc_FILES+=eglibc-ports-$(eglibc_VERSION).tar.gz

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_ports_add_on:
	$(call PRINT_INFO,adding eglibc ports)
	@cd $(@D)/eglibc-$(eglibc_VERSION) && \
		$(TAR) --no-same-owner -xzf $(eglibc_SRC_DL_DIR)/eglibc-ports-$(eglibc_VERSION).tar.gz && \
		mv eglibc-ports-$(eglibc_VERSION) ports
	@touch $@
eglibc-ports-addon: eglibc-fetch $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_ports_add_on
eglibc-unpack: eglibc-ports-addon
endif

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_configure_option_groups:
	$(call PRINT_INFO,configuring eglibc option groups to default to off)
	$(SED) -i -re 's/y/n/g' $(eglibc_DDIR)/option-groups.defaults
	@touch $@

# if we need to rewrite the host libc.so file for the host path
#@$(SED) -i -r -e "s,/usr/lib,.," -e "s,/lib/,./,g" $(TSWO_HOST_INSTALL_DIR)/lib/libc.so
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage2_built:
	$(call PRINT_INFO,building eglibc stage2)
	@mkdir -p $(@D)/eglibc-$(eglibc_VERSION)-build
	grep 'TSWO_EGLIBC_OPTION_' $(BASE_DIR)/.config | $(SED) -re 's/TSWO_EGLIBC_//g' > $(@D)/eglibc-$(eglibc_VERSION)-build/option-groups.config
	cd $(@D)/eglibc-$(eglibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
		$(eglibc_BUILD_OPTIONS) \
		$(@D)/eglibc-$(eglibc_VERSION)/configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
			--prefix=/usr \
			--with-headers=$(TSWO_HOST_INSTALL_DIR)/usr/include \
			--disable-profile --without-gd --without-cvs --enable-add-ons \
			$(eglibc_CONFIGURE_OPTIONS)
	cd $(@D)/eglibc-$(eglibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD)
	cd $(@D)/eglibc-$(eglibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD) install install_root=$(TSWO_HOST_INSTALL_DIR)
	@touch $(@)

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage1_built:
	$(call PRINT_INFO,building eglibc stage1)
	@if [ -d $(BUILD_DIR)/eglibc-headers ]; then rm -rf $(BUILD_DIR)/eglibc-headers;fi
	@mkdir $(BUILD_DIR)/eglibc-headers
	@(if [ ! -d $(TSWO_HOST_INSTALL_DIR)/usr ];then cd $(TSWO_HOST_INSTALL_DIR) && ln -s . usr;fi)
	grep 'TSWO_EGLIBC_OPTION_' $(BASE_DIR)/.config | $(SED) -re 's/TSWO_EGLIBC_//g' > $(BUILD_DIR)/eglibc-headers/option-groups.config
	cd $(BUILD_DIR)/eglibc-headers && \
		PATH=$(BUILD_PATH) \
		$(TARGET_ALIASES) \
		$(TARGET_BUILD_OPTIONS) \
		$(eglibc_BUILD_OPTIONS) \
		$(@D)/eglibc-$(eglibc_VERSION)/configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
			--prefix=/usr \
			--with-headers=$(TSWO_HOST_INSTALL_DIR)/usr/include \
			--disable-profile --without-gd --without-cvs --enable-add-ons
	cd $(BUILD_DIR)/eglibc-headers && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD) csu/subdir_lib $(eglibc_BUILD_OPTIONS) && \
		cp csu/crt1.o csu/crti.o csu/crtn.o $(TSWO_HOST_INSTALL_DIR)/usr/lib
	$(TSWO_HOST_INSTALL_DIR)/bin/$(TSWO_FULL_TARGET_NAME)-gcc -nostdlib -nostartfiles -shared -x c /dev/null \
		-o $(TSWO_HOST_INSTALL_DIR)/usr/lib/libc.so
	cd $(BUILD_DIR)/eglibc-headers && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD) install-headers install_root=$(TSWO_HOST_INSTALL_DIR) \
			install-bootstrap-headers=yes
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)/usr/include/{gnu,bits}
	cp $(@D)/eglibc-$(eglibc_VERSION)/include/gnu/stubs.h $(TSWO_HOST_INSTALL_DIR)/usr/include/gnu/
	cp $(BUILD_DIR)/eglibc-headers/bits/stdio_lim.h $(TSWO_HOST_INSTALL_DIR)/usr/include/bits/
	@touch $(@)

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_host_installed: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage2_built
	@touch $(@)

#@$(SED) -i -r -e "s,/usr/lib,.," -e "s,/lib/,../../,g" $(TSWO_RFS_INSTALL_DIR)/usr/lib/libc.so
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_rfs_installed: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage2_built
	$(call PRINT_INFO,installing eglibc $(eglibc_VERSION) to the target)
	cd $(@D)/eglibc-$(eglibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD) install install_root=$(TSWO_RFS_INSTALL_DIR)
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/*.o
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/libc.so
	@touch $(@)

ifeq (y,$(strip $(TSWO_INSTALL_TOOLCHAIN_TO_TARGET)))
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_rfs_installed: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_libc_so_rfs_installed
endif

# this may not always work... just call install again instead
#$(eglibc_MAKE_CMD) $(TSWO_RFS_INSTALL_DIR)/usr/lib/libc.so install_root=$(TSWO_RFS_INSTALL_DIR)
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_libc_so_rfs_installed: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_rfs_installed
	$(call PRINT_INFO,installing libc.so script to the target)
	cd $(@D)/eglibc-$(eglibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD) install install_root=$(TSWO_RFS_INSTALL_DIR)
	@touch $@

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_packaged: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage2_built
	$(call PRINT_INFO,packaging eglibc $(eglibc_VERSION))
	cd $(@D)/eglibc-$(eglibc_VERSION)-build && \
		PATH=$(BUILD_PATH) \
		$(eglibc_MAKE_CMD) install install_root=$(eglibc_PKG_INSTALL_DIR)
	@touch $@

ifeq (y,$(strip $(TSWO_gcc_REQUIRES_PRECONFIGURE_HEADERS)))
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_preconfigured:
	@(if [ ! -d $(TSWO_HOST_INSTALL_DIR)/usr ];then cd $(TSWO_HOST_INSTALL_DIR) && ln -s . usr;fi)
	@mkdir -p $(@D)/preconfigure
	grep 'TSWO_EGLIBC_OPTION_' $(BASE_DIR)/.config | $(SED) -re 's/TSWO_EGLIBC_//g' > $(@D)/preconfigure/option-groups.config
	cd $(@D)/preconfigure && \
		$(@D)/eglibc-$(eglibc_VERSION)/configure \
			--target=$(TSWO_FULL_TARGET_NAME) \
			--host=$(TSWO_FULL_TARGET_NAME) \
			--build=$(TSWO_BUILD_NAME) \
			--prefix=/usr \
			--with-headers=$(TSWO_HOST_INSTALL_DIR)/usr/include \
			--disable-profile --without-gd --without-cvs --enable-add-ons
	cd $(@D)/preconfigure && \
		$(eglibc_MAKE_CMD) install-headers install_root=$(TSWO_HOST_INSTALL_DIR) \
			install-bootstrap-headers=yes
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)/usr/include/{gnu,bits}
	cp $(@D)/eglibc-$(eglibc_VERSION)/include/gnu/stubs.h $(TSWO_HOST_INSTALL_DIR)/usr/include/gnu/
	cp $(@D)/preconfigure/bits/stdio_lim.h $(TSWO_HOST_INSTALL_DIR)/usr/include/bits/
	@touch $(@)
else
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_preconfigured:
	@touch $(@)
endif

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_configured:
	@touch $(@)

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_built:
	@touch $(@)

$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_rfs_installed: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_rfs_installed
	@touch $@

# after stage2, host install we cleanup the target binaries added to our host toolchain
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_host_installed: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_host_installed $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage2_built
	@rm -f $(TSWO_HOST_INSTALL_DIR)/bin/{gencat,getconf,getent,iconv,locale,localedef,rpcgen,sprof,pcprofiledump}
	@rm -f $(TSWO_HOST_INSTALL_DIR)/sbin/{sln,zdump,zic,iconvconfig,nscd,ldconfig,rpcinfo}
	@touch $@

# install the libc from a binary toolchain
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_binary_rfs_installed:
	$(call PRINT_INFO,installing eglibc $(eglibc_VERSION) to the target)
	mkdir -p $(TSWO_RFS_INSTALL_DIR)/{lib,usr/lib}
	@mkdir -p $(@D)
	cp -frP $(BINARY_TOOLCHAIN_COPY_DIR)/libc/* $(TSWO_RFS_INSTALL_DIR)/lib/
	mv -f $(TSWO_RFS_INSTALL_DIR)/lib/libc.so $(TSWO_RFS_INSTALL_DIR)/usr/lib/
	mv -f $(TSWO_RFS_INSTALL_DIR)/lib/*.o $(TSWO_RFS_INSTALL_DIR)/usr/lib/
ifneq (y,$(strip $(TSWO_INSTALL_TOOLCHAIN_TO_TARGET)))
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/*.o
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/libc.so
endif
	@touch $@
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_binary_rfs_installed:
	@touch $@
$(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_binary_packaged:
	$(call PRINT_INFO,packaging eglibc $(eglibc_VERSION))
	mkdir -p $(eglibc_PKG_INSTALL_DIR)/{lib,usr/lib}
	@mkdir -p $(@D)
	cp -frP $(BINARY_TOOLCHAIN_COPY_DIR)/libc/* $(eglibc_PKG_INSTALL_DIR)/lib/
	mv -f $(eglibc_PKG_INSTALL_DIR)/lib/libc.so $(eglibc_PKG_INSTALL_DIR)/usr/lib/
	$(call CREATE_PACKAGE, eglibc,$(eglibc_PKG_INSTALL_DIR),$(eglibc_PKG_NAME),$(eglibc_VERSION),$(eglibc_BUILDNUM),$(eglibc_RUNTIME_DEPENDS))
	@rm -rf $(eglibc_PKG_INSTALL_DIR)
	@touch $@

eglibc-configure-option-groups: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_configure_option_groups
eglibc-patch: eglibc-configure-option-groups
eglibc-rfs-install: eglibc-patch $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_rfs_installed $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_rfs_installed
eglibc-host-install: eglibc-patch $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_host_installed $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_host_installed
eglibc-binary-rfs-install: archive_pristine_toolchain_libs $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_binary_rfs_installed $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_binary_rfs_installed
eglibc-binary-package: $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_binary_packaged

eglibc_build_stage2: eglibc-patch $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage2_built $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_post_host_installed
eglibc_build_stage1: eglibc-patch $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_stage1_built
eglibc_preconfigure: eglibc-patch $(BUILD_DIR)/eglibc-$(eglibc_VERSION)/.stamp_preconfigured

