# Copyright (C) 2007-2009 Timesys Corporation
# vim:set noexpandtab: 

$(eval $(call COMMON_SETUP,uClibc))

uClibc-fetch: $(uClibc_SRC_DL_DIR)/uclibc_config

uClibc_SRC_DL_DIR=$(SRC_DL_DIR)/u/uClibc/uClibc-$(uClibc_VERSION)

$(uClibc_SRC_DL_DIR)/uclibc_config:
	$(call PRINT_INFO,fetching uClibc configuration)
	@mkdir -p $(uClibc_SRC_DL_DIR)
	$(call FETCH_REMOTE,$(uClibc_SRC_DL_DIR),$(TSWO_uClibc_CONFIG))
	if [ ! -f $@ ]; then cp -f $(uClibc_SRC_DL_DIR)/$(notdir $(call UNQUOTE_VAR,$(TSWO_uClibc_CONFIG))) $@;fi

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_configured: $(uClibc_SRC_DL_DIR)/uclibc_config
	$(call PRINT_INFO,installing uClibc configuration)
	$(uClibc_MAKE_CMD) KBUILD_HAVE_NLS=no -C $(@D)/uClibc-$(uClibc_VERSION) \
		CROSS= \
		allnoconfig \
		KCONFIG_ALLCONFIG=$(uClibc_SRC_DL_DIR)/uclibc_config
	@touch $(@)

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_stage2_built:
	$(call PRINT_INFO,building uClibc stage2)
	PATH=$(BUILD_PATH) \
		$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			PREFIX= \
			DEVEL_PREFIX=/ \
			RUNTIME_PREFIX=/ \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
			all
	PATH=$(BUILD_PATH) \
		$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			PREFIX=$(TSWO_HOST_INSTALL_DIR) \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
			install_runtime install_dev
	@touch $(@)


# unused at the moment
$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_ldd_and_readelf_built:
	$(call PRINT_INFO,building ldd and readelf)
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) CROSS= distclean
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) CROSS= allnoconfig
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) CROSS= headers KERNEL_HEADERS=/usr/include
	(cd $(@D)/uClibc-$(uClibc_VERSION) && \
		$$HOSTCC -Os -s -I include utils/readelf.c -o "$(TSWO_HOST_INSTALL_DIR)/bin/$(TSWO_FULL_TARGET_NAME)-readelf")
	(cd $(@D)/uClibc-$(uClibc_VERSION) && \
		$$HOSTCC -Os -s -I ldso/include utils/ldd.c -o "$(TSWO_HOST_INSTALL_DIR)/bin/$(TSWO_FULL_TARGET_NAME)-ldd")

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_stage1_built:
	$(call PRINT_INFO,building uClibc stage1)
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)/include
	@mkdir -p $(TSWO_HOST_INSTALL_DIR)/lib
	PATH=$(BUILD_PATH) \
		$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			PREFIX= \
			DEVEL_PREFIX=/ \
			RUNTIME_PREFIX=/ \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
			all
	PATH=$(BUILD_PATH) \
		$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			PREFIX=$(TSWO_HOST_INSTALL_DIR) \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
			install_runtime install_dev
	@touch $(@)


$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_host_installed:
	@touch $(@)



$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_rfs_installed:
	$(call PRINT_INFO,installing uClibc $(uClibc_VERSION) to the target)
	PATH=$(BUILD_PATH) \
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
		all install_runtime install_dev PREFIX=$(TSWO_RFS_INSTALL_DIR) DEVEL_PREFIX=/usr/ RUNTIME_PREFIX=/
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/*.o
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/libc.so
	@touch $(@)

ifeq (y,$(strip $(TSWO_INSTALL_TOOLCHAIN_TO_TARGET)))
$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_post_rfs_installed: $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_libc_so_rfs_installed
endif

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_libc_so_rfs_installed: $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_rfs_installed
	$(call PRINT_INFO,installing uClibc development files to the target)
	PATH=$(BUILD_PATH) \
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
		install_dev PREFIX=$(TSWO_RFS_INSTALL_DIR) DEVEL_PREFIX=/usr/ RUNTIME_PREFIX=/
	@touch $@

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_packaged:
	PATH=$(BUILD_PATH) \
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
			CROSS="$(TSWO_FULL_TARGET_NAME)-" \
			KERNEL_HEADERS="$(TSWO_HOST_INSTALL_DIR)/include" \
		all install_runtime install_dev PREFIX=$(uClibc_PKG_INSTALL_DIR) DEVEL_PREFIX=/usr/ RUNTIME_PREFIX=/
	@touch $(@)


$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_preconfigured:
	$(call PRINT_INFO,preconfiguring uClibc)
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
		PREFIX=$(TSWO_HOST_INSTALL_DIR)/ \
		DEVEL_PREFIX=/ \
		RUNTIME_PREFIX=$(TSWO_HOST_INSTALL_DIR)/ \
		HOSTCC=gcc \
		pregen install_dev
	(cd $(TSWO_HOST_INSTALL_DIR) && if [ ! -e usr ]; then ln -s . usr;fi)
	$(uClibc_MAKE_CMD) -C $(@D)/uClibc-$(uClibc_VERSION) \
		clean
	@touch $(@)

uClibc-rfs-install:  $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_rfs_installed
uClibc-host-install: $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_stage2_built $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_host_installed

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_built:
	@touch $(@)

uclibc-menuconfig: uClibc-patch
	cd $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/uClibc-$(uClibc_VERSION) && \
		if [ -f $(uClibc_SRC_DL_DIR)/uclibc_config ]; then cp $(uClibc_SRC_DL_DIR)/uclibc_config .config;fi && \
		$(uClibc_MAKE_CMD) KBUILD_HAVE_NLS=no menuconfig && \
		if [ -f .config ]; then cp .config $(uClibc_SRC_DL_DIR)/uclibc_config;fi

uClibc-patch: $(uClibc_SRC_DL_DIR)/uclibc_config
uClibc_build_stage2: uClibc-patch $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_configured $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_stage2_built
uClibc_build_stage1: uClibc-patch $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_configured $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_stage1_built
uClibc_preconfigure: uClibc-patch $(uClibc_SRC_DL_DIR)/uclibc_config $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_configured $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_preconfigured
uClibc-configure:    uClibc-patch $(uClibc_SRC_DL_DIR)/uclibc_config $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_configured

uClibc-binary-rfs-install: archive_pristine_toolchain_libs $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_binary_rfs_installed $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_post_binary_rfs_installed
uClibc-binary-package: $(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_binary_packaged


$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_binary_rfs_installed:
	@mkdir -p $(@D)
	mkdir -p $(TSWO_RFS_INSTALL_DIR)/{lib,usr/lib}
	cp -frP $(BINARY_TOOLCHAIN_COPY_DIR)/libc/* $(TSWO_RFS_INSTALL_DIR)/lib/
	mv -f $(TSWO_RFS_INSTALL_DIR)/lib/libc.so $(TSWO_RFS_INSTALL_DIR)/usr/lib/
	mv -f $(TSWO_RFS_INSTALL_DIR)/lib/*.o $(TSWO_RFS_INSTALL_DIR)/usr/lib/
ifneq (y,$(strip $(TSWO_INSTALL_TOOLCHAIN_TO_TARGET)))
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/*.o
	rm -f $(TSWO_RFS_INSTALL_DIR)/usr/lib/libc.so
endif
	@touch $@
$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_post_binary_rfs_installed:
	@mkdir -p $(@D)
	@touch $@

$(BUILD_DIR)/uClibc-$(uClibc_VERSION)/.stamp_binary_packaged:
	$(call PRINT_INFO,packaging uClibc $(uClibc_VERSION))
	mkdir -p $(uClibc_PKG_INSTALL_DIR)/{lib,usr/lib}
	@mkdir -p $(@D)
	cp -frP $(BINARY_TOOLCHAIN_COPY_DIR)/libc/* $(uClibc_PKG_INSTALL_DIR)/lib/
	mv -f $(uClibc_PKG_INSTALL_DIR)/lib/libc.so $(uClibc_PKG_INSTALL_DIR)/usr/lib/
	$(call CREATE_PACKAGE, uClibc,$(uClibc_PKG_INSTALL_DIR),$(uClibc_PKG_NAME),$(uClibc_VERSION),$(uClibc_BUILDNUM),$(uClibc_RUNTIME_DEPENDS))
	@rm -rf $(uClibc_PKG_INSTALL_DIR)
	@touch $@

